@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/kernel/ioctl.rs: ioctl(2) handler
 //
-// Copyright (c) 2023, 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2023, 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
@@ -37,15 +37,18 @@ pub(crate) fn sys_ioctl(request: UNotifyEventRequest) -> ScmpNotifResp {
                 .ok()
                 .flatten();
             let grp = cap.to_string().to_ascii_lowercase();
+            let tip = if let Some(name) = ctl.as_ref().and_then(|ctl| ctl.get(0)) {
+                format!("configure `allow/{grp}+{name}'")
+            } else {
+                format!("configure `allow/{grp}+{arg:#x}'")
+            };
             if log_scmp {
                 warn!("ctx": "access", "cap": cap, "act": action,
-                    "sys": "ioctl", "ctl": ctl,
-                    "tip": format!("configure `allow/{grp}+{arg:#x}'"),
+                    "sys": "ioctl", "ctl": ctl, "tip": tip,
                     "req": &request);
             } else {
                 warn!("ctx": "access", "cap": cap, "act": action,
-                    "sys": "ioctl", "ctl": ctl,
-                    "tip": format!("configure `allow/{grp}+{arg:#x}'"),
+                    "sys": "ioctl", "ctl": ctl, "tip": tip,
                     "pid": request.scmpreq.pid);
             }
         }
