@@ -14,7 +14,9 @@ use nix::{
 };
 
 use crate::{
-    config::MMAP_MIN_ADDR, confine::scmp_arch_bits, fs::peer_inode, req::UNotifyEventRequest,
+    confine::{is_valid_ptr, scmp_arch_bits},
+    fs::peer_inode,
+    req::UNotifyEventRequest,
 };
 
 pub(crate) fn handle_getsockopt(
@@ -32,7 +34,7 @@ pub(crate) fn handle_getsockopt(
     let optlen_ptr = args[4];
 
     // optlen pointer must not be NULL.
-    if optlen_ptr < *MMAP_MIN_ADDR {
+    if !is_valid_ptr(optlen_ptr, request.scmpreq.data.arch) {
         return Err(Errno::EFAULT);
     }
 
@@ -45,7 +47,7 @@ pub(crate) fn handle_getsockopt(
 
     // Read *optlen from tracee.
     let mut len_buf = [0u8; SIZEOF_SOCKLEN_T];
-    let read = request.read_mem(&mut len_buf, optlen_ptr)?;
+    let read = request.read_mem(&mut len_buf, optlen_ptr, SIZEOF_SOCKLEN_T)?;
     if read != SIZEOF_SOCKLEN_T {
         return Err(Errno::EINVAL);
     }
@@ -54,7 +56,7 @@ pub(crate) fn handle_getsockopt(
     let orig_optlen = u32::from_ne_bytes([len_buf[0], len_buf[1], len_buf[2], len_buf[3]]) as usize;
 
     // If optval == NULL but *optlen > 0, kernel returns EFAULT.
-    if optval_ptr < *MMAP_MIN_ADDR && orig_optlen > 0 {
+    if !is_valid_ptr(optval_ptr, request.scmpreq.data.arch) && orig_optlen > 0 {
         return Err(Errno::EFAULT);
     }
 
