@@ -25,7 +25,8 @@ use crate::{
     },
     config::MAX_RW_COUNT,
     confine::scmp_arch_bits,
-    fs::{get_nonblock, has_recv_timeout, is_same_vm},
+    fd::{get_nonblock, has_recv_timeout},
+    fs::is_same_vm,
     kernel::{
         net::{canon_addr, get_addr, make_lo6addr, make_loaddr, sandbox_addr, to_msgflags},
         sandbox_path,
@@ -166,6 +167,7 @@ pub(crate) fn handle_sendmsg(
         None
     };
 
+    #[allow(unused)]
     if let Some((
         ref control_fds,
         ref control_creds,
@@ -238,6 +240,7 @@ pub(crate) fn handle_sendmsg(
             control_messages.push(ControlMessage::AlgSetAeadAssoclen(assoclen));
         }
 
+        #[cfg(not(target_os = "android"))]
         for gso_segments in control_udp_gso_segments {
             control_messages.push(ControlMessage::UdpGsoSegments(gso_segments));
         }
@@ -254,6 +257,7 @@ pub(crate) fn handle_sendmsg(
             control_messages.push(ControlMessage::RxqOvfl(rxq_ovfl));
         }
 
+        #[cfg(not(target_os = "android"))]
         for tx_time in control_tx_times {
             control_messages.push(ControlMessage::TxTime(tx_time));
         }
@@ -546,6 +550,7 @@ pub(crate) fn handle_sendmmsg(
 
         let mut control_messages = Vec::new();
         for control_datum in &control_data {
+            #[allow(unused)]
             if let Some((
                 ref control_fds,
                 ref control_creds,
@@ -619,6 +624,7 @@ pub(crate) fn handle_sendmmsg(
                     control_messages.push(ControlMessage::AlgSetAeadAssoclen(assoclen));
                 }
 
+                #[cfg(not(target_os = "android"))]
                 for gso_segments in control_udp_gso_segments {
                     control_messages.push(ControlMessage::UdpGsoSegments(gso_segments));
                 }
@@ -635,6 +641,7 @@ pub(crate) fn handle_sendmmsg(
                     control_messages.push(ControlMessage::RxqOvfl(rxq_ovfl));
                 }
 
+                #[cfg(not(target_os = "android"))]
                 for tx_time in control_tx_times {
                     control_messages.push(ControlMessage::TxTime(tx_time));
                 }
@@ -815,10 +822,12 @@ fn parse_control_messages(
     let mut control_ivs = Vec::new();
     let mut control_ops = Vec::new();
     let mut control_aead_assoclens = Vec::new();
+    #[allow(unused)]
     let mut control_udp_gso_segments = Vec::new();
     let mut control_ipv4_packet_infos = Vec::new();
     let mut control_ipv6_packet_infos = Vec::new();
     let mut control_rxq_ovfls = Vec::new();
+    #[allow(unused)]
     let mut control_tx_times = Vec::new();
     let mut control_ipv4_toses = Vec::new();
     let mut control_ipv6_tclasses = Vec::new();
@@ -988,6 +997,7 @@ fn parse_control_messages(
                 let assoclen = unsafe { *(data.as_ptr() as *const u32) };
                 control_aead_assoclens.push(assoclen); // Store the AEAD assoclen
             }
+            #[cfg(not(target_os = "android"))]
             (libc::SOL_UDP, libc::UDP_SEGMENT) => {
                 // SAFETY: This unsafe block casts a pointer from `*const u8` to `*const
                 // u16` under the assumption that the data at `data.as_ptr()` is correctly
@@ -1070,6 +1080,7 @@ fn parse_control_messages(
                 let rxq_ovfl = unsafe { *(data.as_ptr() as *const u32) };
                 control_rxq_ovfls.push(rxq_ovfl); // Store the Rx queue overflow count
             }
+            #[cfg(not(target_os = "android"))]
             (libc::SOL_SOCKET, libc::SCM_TXTIME) => {
                 // SAFETY: The casting from `*const u8` to `*const u64` is contingent upon
                 // the assumption that the `data` buffer is aligned according to `u64`
