@@ -15,7 +15,8 @@ use nix::{
 
 use crate::{
     compat::MsgFlags,
-    fs::{get_nonblock, has_recv_timeout},
+    config::MAX_RW_COUNT,
+    fd::{get_nonblock, has_recv_timeout},
     kernel::net::to_msgflags,
     req::UNotifyEventRequest,
 };
@@ -46,7 +47,7 @@ pub(crate) fn handle_sendto(
     // 3. Buffer read from sandbox process MUST be zeroized on drop.
     let len = usize::try_from(args[2])
         .or(Err(Errno::EINVAL))?
-        .min(1000000); // Cap count at 1mio.
+        .min(*MAX_RW_COUNT); // Cap count at MAX_RW_COUNT.
 
     // read_vec_zeroed returns an empty Vec with len==0,
     // without performing any memory reads.
