@@ -163,7 +163,7 @@ pub(crate) fn handle_socketpair(
 
     // SAFETY: Check pointer against mmap_min_addr.
     let fdptr = args[3];
-    if !is_valid_ptr(fdptr) {
+    if !is_valid_ptr(fdptr, request.scmpreq.data.arch) {
         return Err(Errno::EFAULT);
     }
 
