@@ -10,7 +10,7 @@ use bitflags::bitflags;
 use libseccomp::ScmpNotifResp;
 use nix::{
     errno::Errno,
-    unistd::{getpgid, getpgrp, Pid},
+    unistd::{getpgrp, Pid},
 };
 
 use crate::req::UNotifyEventRequest;
@@ -143,35 +143,15 @@ fn syscall_signal_handler(request: UNotifyEventRequest, syscall: SignalSyscall)
         return request.fail_syscall(Errno::EINVAL);
     }
 
-    // Guard syd tasks.
+    // No need to check for group signals directed at the current
+    // process group here because Syd does not share the process group
+    // with the sandbox process as of 3.48.0.
     //
     // pid <=0 only for kill here.
-    match pid {
-        0 => {
-            // SAFETY: Guard against group signals.
-            // kill(0, 9) -> Send signal to _current_ process group.
-            match getpgid(Some(req.pid())) {
-                Ok(pgrp) if pgrp == getpgrp() => {
-                    // SAFETY: This is a version of killpg().
-                    // We must stop this signal if Syd is in
-                    // the same process group as the process, otherwise
-                    // continue is safe. EACCES is no further
-                    // information leak as sig==0 is pass-through.
-                    return request.fail_syscall(Errno::EACCES);
-                }
-                Err(_) => {
-                    // ESRCH is no further information leak as
-                    // sig==0 is pass-through.
-                    return request.fail_syscall(Errno::ESRCH);
-                }
-                _ => {}
-            }
-        }
-        -1 => {
-            // SAFETY: We do not allow mass signaling with -1.
-            return request.fail_syscall(Errno::EACCES);
-        }
-        _ => {}
+    if pid == -1 {
+        // SAFETY: We do not allow mass signaling with -1.
+        // ESRCH errno(3) is consistent with landlock(7) scoped signals.
+        return request.fail_syscall(Errno::ESRCH);
     }
 
     // kill and sigqueue support negative PIDs.
@@ -180,35 +160,35 @@ fn syscall_signal_handler(request: UNotifyEventRequest, syscall: SignalSyscall)
     // Check for Syd tasks.
     let syd = Pid::this().as_raw();
 
-    // SAFETY: Note, we deny with EACCES, rather than returning success
+    // SAFETY: Note, we deny with EPERM, rather than returning success
     // because we have a kernel-level bpf filter that _allows_ the
     // respective signaler system call _only when_ the signal is 0.
     // Therefore we're not leaking any further information by returning
-    // EACCES here.
+    // EPERM here.
     if !syscall.is_thread() && syd == pid_abs {
-        return request.fail_syscall(Errno::EACCES);
+        return request.fail_syscall(Errno::EPERM);
     }
 
     if syscall.is_thread() && syd == pid {
-        return request.fail_syscall(Errno::EACCES);
+        return request.fail_syscall(Errno::EPERM);
     }
 
     if syscall.is_thread_group() && syd == tid {
-        return request.fail_syscall(Errno::EACCES);
+        return request.fail_syscall(Errno::EPERM);
     }
 
     if !syscall.is_thread()
         // SAFETY: Check for Syd threads with the abstract PID.
         && Errno::result(unsafe { libc::syscall(libc::SYS_tgkill, syd, pid_abs, 0) }).is_ok()
     {
-        return request.fail_syscall(Errno::EACCES);
+        return request.fail_syscall(Errno::EPERM);
     }
 
     if syscall.is_thread()
         // SAFETY: Check for Syd threads with the PID.
         && Errno::result(unsafe { libc::syscall(libc::SYS_tgkill, syd, pid, 0) }).is_ok()
     {
-        return request.fail_syscall(Errno::EACCES);
+        return request.fail_syscall(Errno::EPERM);
     }
 
     if syscall.is_thread_group()
@@ -219,19 +199,19 @@ fn syscall_signal_handler(request: UNotifyEventRequest, syscall: SignalSyscall)
         )
         .is_ok()
     {
-        return request.fail_syscall(Errno::EACCES);
+        return request.fail_syscall(Errno::EPERM);
     }
 
     // Check signals directed to Syd's process group.
     let syd_pgid = getpgrp().as_raw();
     if !syscall.is_thread() && syd_pgid == pid_abs {
-        return request.fail_syscall(Errno::EACCES);
+        return request.fail_syscall(Errno::EPERM);
     }
     if syscall.is_thread() && syd_pgid == pid {
-        return request.fail_syscall(Errno::EACCES);
+        return request.fail_syscall(Errno::EPERM);
     }
     if syscall.is_thread_group() && syd_pgid == tid {
-        return request.fail_syscall(Errno::EACCES);
+        return request.fail_syscall(Errno::EPERM);
     }
 
     // SAFETY: This is safe because we haven't dereferenced
