@@ -9,13 +9,14 @@
 // SAFETY: This module has been liberated from unsafe code!
 #![forbid(unsafe_code)]
 
-use std::sync::LazyLock;
+use std::{fmt, sync::LazyLock};
 
 use memchr::arch::all::is_equal;
 use nix::{
     errno::Errno,
     fcntl::{OFlag, AT_FDCWD},
 };
+use serde::{ser::SerializeMap, Serialize, Serializer};
 
 use crate::{
     compat::{openat2, OpenHow, ResolveFlag},
@@ -745,6 +746,149 @@ impl LandlockPolicy {
     }
 }
 
+impl fmt::Display for LandlockPolicy {
+    #[expect(clippy::cognitive_complexity)]
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        writeln!(f, "  Landlock Policy:")?;
+
+        let level = self.compat_level.unwrap_or(CompatLevel::BestEffort);
+        writeln!(f, "    Compat Level: {level:?}")?;
+
+        writeln!(f, "    Scoped Abstract Unix Sockets: {}", self.scoped_abs)?;
+        writeln!(f, "    Scoped Signals: {}", self.scoped_sig)?;
+        writeln!(f, "    Restrict Self Flags: {:?}", self.restrict_self_flags)?;
+
+        macro_rules! fmt_pathset {
+            ($field:ident, $name:expr) => {
+                if let Some(ref set) = self.$field {
+                    if !set.is_empty() {
+                        let mut paths: Vec<_> = set.iter().collect();
+                        paths.sort();
+                        writeln!(f, "    {}: {paths:?}", $name)?;
+                    }
+                }
+            };
+        }
+
+        macro_rules! fmt_portset {
+            ($field:ident, $name:expr) => {
+                if let Some(ref set) = self.$field {
+                    if set.count_ones(..) > 0 {
+                        let mut ports: Vec<_> = set.ones().collect();
+                        ports.sort();
+                        writeln!(f, "    {}: {ports:?}", $name)?;
+                    }
+                }
+            };
+        }
+
+        fmt_pathset!(read_pathset, "Read Pathset");
+        fmt_pathset!(write_pathset, "Write Pathset");
+        fmt_pathset!(exec_pathset, "Exec Pathset");
+        fmt_pathset!(ioctl_pathset, "Ioctl Pathset");
+        fmt_pathset!(create_pathset, "Create Pathset");
+        fmt_pathset!(delete_pathset, "Delete Pathset");
+        fmt_pathset!(rename_pathset, "Rename Pathset");
+        fmt_pathset!(symlink_pathset, "Symlink Pathset");
+        fmt_pathset!(truncate_pathset, "Truncate Pathset");
+        fmt_pathset!(readdir_pathset, "Readdir Pathset");
+        fmt_pathset!(mkdir_pathset, "Mkdir Pathset");
+        fmt_pathset!(rmdir_pathset, "Rmdir Pathset");
+        fmt_pathset!(mkbdev_pathset, "Mkbdev Pathset");
+        fmt_pathset!(mkcdev_pathset, "Mkcdev Pathset");
+        fmt_pathset!(mkfifo_pathset, "Mkfifo Pathset");
+        fmt_pathset!(bind_pathset, "Bind Pathset");
+        fmt_portset!(bind_portset, "  Bind Portset");
+        fmt_portset!(conn_portset, "  Connect Portset");
+
+        Ok(())
+    }
+}
+
+impl Serialize for LandlockPolicy {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        let mut map = serializer.serialize_map(None)?;
+
+        let level = match self.compat_level.unwrap_or(CompatLevel::BestEffort) {
+            CompatLevel::BestEffort => "best-effort",
+            CompatLevel::SoftRequirement => "soft-requirement",
+            CompatLevel::HardRequirement => "hard-requirement",
+        };
+        map.serialize_entry("compat_level", level)?;
+
+        map.serialize_entry("scoped_abs", &self.scoped_abs)?;
+        map.serialize_entry("scoped_sig", &self.scoped_sig)?;
+
+        let mut flags = Vec::new();
+        if self
+            .restrict_self_flags
+            .contains(RestrictSelfFlags::LOG_SAME_EXEC_OFF)
+        {
+            flags.push("log-same-exec-off");
+        }
+        if self
+            .restrict_self_flags
+            .contains(RestrictSelfFlags::LOG_NEW_EXEC_ON)
+        {
+            flags.push("log-new-exec-on");
+        }
+        if self
+            .restrict_self_flags
+            .contains(RestrictSelfFlags::LOG_SUBDOMAINS_OFF)
+        {
+            flags.push("log-subdomains-off");
+        }
+        map.serialize_entry("restrict_self_flags", &flags)?;
+
+        let mut ser_pathset = |name: &str, set: &Option<PathSet>| -> Result<(), S::Error> {
+            if let Some(ref set) = set {
+                if !set.is_empty() {
+                    let mut paths: Vec<_> = set.iter().map(|p| p.to_string()).collect();
+                    paths.sort();
+                    map.serialize_entry(name, &paths)?;
+                }
+            }
+            Ok(())
+        };
+
+        ser_pathset("read_pathset", &self.read_pathset)?;
+        ser_pathset("write_pathset", &self.write_pathset)?;
+        ser_pathset("exec_pathset", &self.exec_pathset)?;
+        ser_pathset("ioctl_pathset", &self.ioctl_pathset)?;
+        ser_pathset("create_pathset", &self.create_pathset)?;
+        ser_pathset("delete_pathset", &self.delete_pathset)?;
+        ser_pathset("rename_pathset", &self.rename_pathset)?;
+        ser_pathset("symlink_pathset", &self.symlink_pathset)?;
+        ser_pathset("truncate_pathset", &self.truncate_pathset)?;
+        ser_pathset("readdir_pathset", &self.readdir_pathset)?;
+        ser_pathset("mkdir_pathset", &self.mkdir_pathset)?;
+        ser_pathset("rmdir_pathset", &self.rmdir_pathset)?;
+        ser_pathset("mkbdev_pathset", &self.mkbdev_pathset)?;
+        ser_pathset("mkcdev_pathset", &self.mkcdev_pathset)?;
+        ser_pathset("mkfifo_pathset", &self.mkfifo_pathset)?;
+        ser_pathset("bind_pathset", &self.bind_pathset)?;
+
+        let mut ser_portset = |name: &str, set: &Option<PortSet>| -> Result<(), S::Error> {
+            if let Some(ref set) = set {
+                if set.count_ones(..) > 0 {
+                    let mut ports: Vec<_> = set.ones().collect();
+                    ports.sort();
+                    map.serialize_entry(name, &ports)?;
+                }
+            }
+            Ok(())
+        };
+
+        ser_portset("bind_portset", &self.bind_portset)?;
+        ser_portset("conn_portset", &self.conn_portset)?;
+
+        map.end()
+    }
+}
+
 // syd::landlock::path_beneath_rules tailored for Syd use-case.
 #[expect(clippy::cognitive_complexity)]
 #[expect(clippy::disallowed_methods)]
