@@ -111,10 +111,14 @@ pub mod api;
 /// Assembly instruction decoder
 #[cfg(feature = "asm")]
 pub mod asm;
+/// Cgroup v2 management for resource limits
+pub mod cgroup;
 /// System call argument cookies
 pub mod cookie;
 /// Utilities to mask sensitive information in proc files
 pub(crate) mod mask;
+/// Safe mount interface
+pub mod mount;
 /// System call handlers
 #[macro_use]
 pub(crate) mod kernel;
@@ -136,6 +140,8 @@ pub mod dns;
 pub mod elf;
 /// Error types and error handling code.
 pub mod err;
+/// File descriptor utilities
+pub mod fd;
 /// Filesystem utilities
 pub mod fs;
 /// Utilities for hashing
@@ -193,6 +199,7 @@ pub mod uts;
 #[expect(clippy::arithmetic_side_effects)]
 pub mod wildmatch;
 /// Interface to wordexp(3)
+#[cfg(not(target_os = "android"))]
 pub mod wordexp;
 /// Extended attribute utilities
 pub mod xattr;
@@ -229,7 +236,6 @@ use nix::{
     errno::Errno,
     sched::CloneFlags,
     sys::{
-        personality::Persona,
         resource::{getrlimit, Resource},
         signal::{sigaction, signal, SaFlags, SigAction, SigHandler, SigSet, Signal},
         socket::{socket, AddressFamily, SockFlag, SockType},
@@ -242,8 +248,9 @@ use serde::{Serialize, Serializer};
 use crate::{
     compat::{fstatx, lsm_list_modules},
     confine::{
-        check_cross_memory_attach, check_unix_diag, is_coredump, lock_enabled, ns_enabled,
-        seccomp_arch_native_name, vdso_list_calls, SydPersona,
+        apparmor_enabled, check_cross_memory_attach, check_unix_diag, is_coredump, lock_enabled,
+        ns_enabled, seccomp_arch_native_name, selinux_enabled, selinux_enforced, vdso_list_calls,
+        SydArch, SydPersona, SCMP_ARCH,
     },
     err::err2no,
     hash::{aes_ctr_info, check_setsockopt_serial_support, hmac_sha256_info, key_ring_validate},
@@ -261,10 +268,6 @@ use crate::{
 macro_rules! main {
     { $($body:tt)* } => {
         fn main() -> std::process::ExitCode {
-            // Silence mimalloc warnings.
-            #[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
-            syd::config_mimalloc();
-
             match (|| -> syd::err::SydResult<std::process::ExitCode> { $($body)* })() {
                 Ok(code) => code,
                 Err(err) => {
@@ -324,6 +327,19 @@ pub fn syd_info(verbose: bool) -> Result<(), Errno> {
         return Ok(());
     }
 
+    let alloc = if cfg!(all(
+        not(feature = "prof"),
+        target_page_size_4k,
+        target_pointer_width = "64"
+    )) {
+        "GrapheneOS"
+    } else if cfg!(feature = "prof") {
+        "TCMalloc"
+    } else {
+        "Libc"
+    };
+    printfln!("Allocator: {alloc}")?;
+
     let libapi = libseccomp::get_api();
     match ScmpVersion::current() {
         Ok(libver) => {
@@ -451,6 +467,27 @@ pub fn syd_info(verbose: bool) -> Result<(), Errno> {
     };
     printfln!("LSMs: {lsms}.")?;
 
+    // Log SELinux and Apparmor status.
+    let selinux = match selinux_enabled() {
+        Ok(true) => {
+            let enforce = if selinux_enforced().unwrap_or(false) {
+                "Enforcing"
+            } else {
+                "Permissive"
+            };
+            format!("enabled ({enforce})")
+        }
+        Ok(false) => "disabled".to_string(),
+        Err(errno) => format!("? (error: {errno})"),
+    };
+    let apparmor = match apparmor_enabled() {
+        Ok(true) => "enabled".to_string(),
+        Ok(false) => "disabled".to_string(),
+        Err(errno) => format!("? (error: {errno})"),
+    };
+    printfln!("SELinux is {selinux}.")?;
+    printfln!("AppArmor is {apparmor}.")?;
+
     // List vDSO calls.
     match vdso_list_calls() {
         Ok(names) if names.is_empty() => printfln!("No vDSO calls found.")?,
@@ -462,7 +499,7 @@ pub fn syd_info(verbose: bool) -> Result<(), Errno> {
                 .join(", ");
             printfln!("List of vDSO calls: {names}.")?;
         }
-        Err(error) => printfln!("List of vDSO calls: ? (error: {error}")?,
+        Err(error) => printfln!("List of vDSO calls: ? (error: {error}).")?,
     }
 
     // Print information on open file limits.
@@ -487,10 +524,19 @@ pub fn syd_info(verbose: bool) -> Result<(), Errno> {
     // Print detected host Linux kernel version and related features.
     printf!("Host Linux: {}.{} with", KERNEL_VERSION.0, KERNEL_VERSION.1)?;
     printf!(" mmap_min_addr={}", *MMAP_MIN_ADDR)?;
+    printf!(", page_size={}", *PAGE_SIZE)?;
     printf!(
         ", {}at_execve_check",
         if *HAVE_AT_EXECVE_CHECK { "+" } else { "-" }
     )?;
+    printf!(
+        ", {}landlock_scoped_signals",
+        if *HAVE_LANDLOCK_SCOPED_SIGNALS {
+            "+"
+        } else {
+            "-"
+        }
+    )?;
     printf!(
         ", {}madv_guard_install",
         if *HAVE_MADV_GUARD { "+" } else { "-" }
@@ -534,8 +580,8 @@ pub fn syd_info(verbose: bool) -> Result<(), Errno> {
 
     // SAFETY: In libc we trust.
     // Note: nix version truncates unknown bits which we don't want.
-    let pers = match Errno::result(unsafe { libc::personality(0xFFFFFFFF) }) {
-        Ok(n) => SydPersona(Persona::from_bits_retain(n)).to_string(),
+    let pers = match SydPersona::get() {
+        Ok(pers) => pers.to_string(),
         Err(errno) => format!("? (error: {errno})"),
     };
 
@@ -545,6 +591,41 @@ pub fn syd_info(verbose: bool) -> Result<(), Errno> {
         env!("SYD_TARGET_POINTER_WIDTH")
     )?;
 
+    // Log supported architectures.
+    let arch = SCMP_ARCH
+        .iter()
+        .map(SydArch::from)
+        .map(|arch| arch.to_string())
+        .collect::<Vec<_>>()
+        .join(", ");
+    printfln!("Architectures: {arch}")?;
+
+    // Log architectures with ipc(2), socketcall(2) multiplexer support.
+    let mut has_ipc = Vec::new();
+    let mut has_socketcall = Vec::new();
+    for arch in SCMP_ARCH.iter().map(SydArch::from) {
+        if arch.has_ipc() {
+            has_ipc.push(arch.to_string());
+        }
+        if arch.has_socketcall() {
+            has_socketcall.push(arch.to_string());
+        }
+    }
+    if !has_ipc.is_empty() {
+        let plurals = if has_ipc.len() > 1 { "s" } else { "" };
+        let verbone = if has_ipc.len() == 1 { "s" } else { "" };
+        let has_ipc = has_ipc.join(", ");
+        printfln!("Architecture{plurals} {has_ipc} support{verbone} ipc(2) multiplexer.")?;
+    }
+    if !has_socketcall.is_empty() {
+        let plurals = if has_socketcall.len() > 1 { "s" } else { "" };
+        let verbone = if has_socketcall.len() == 1 { "s" } else { "" };
+        let has_socketcall = has_socketcall.join(", ");
+        printfln!(
+            "Architecture{plurals} {has_socketcall} support{verbone} socketcall(2) multiplexer."
+        )?;
+    }
+
     printfln!(
         "CPU: {} ({} cores), {}-endian",
         num_cpus::get(),
@@ -1165,40 +1246,6 @@ pub fn set_sigpipe_dfl() -> Result<(), Errno> {
     unsafe { signal(Signal::SIGPIPE, SigHandler::SigDfl) }.map(drop)
 }
 
-/// Configure mimalloc to silence stderr.
-///
-/// This function is NOT thread-safe, call it early at init.
-#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
-#[allow(non_upper_case_globals)]
-pub fn config_mimalloc() {
-    // Extended feature is not usable (breaks on loongarch64),
-    // therefore we define a subset of mimalloc API here.
-    #[allow(non_camel_case_types)]
-    type mi_option_t = libc::c_int;
-    #[allow(non_camel_case_types)]
-    type mi_output_fun =
-        Option<unsafe extern "C" fn(msg: *const libc::c_char, arg: *mut libc::c_void)>;
-
-    #[link(name = "mimalloc")]
-    extern "C" {
-        fn mi_option_set_enabled(option: mi_option_t, enable: bool);
-        fn mi_register_output(out: mi_output_fun, arg: *mut libc::c_void);
-    }
-    extern "C" fn syd_noop_output(_msg: *const libc::c_char, _arg: *mut libc::c_void) {}
-
-    const mi_option_show_errors: mi_option_t = 0;
-    const mi_option_show_stats: mi_option_t = 1;
-    const mi_option_verbose: mi_option_t = 2;
-
-    // SAFETY: In mimalloc we trust.
-    unsafe {
-        mi_option_set_enabled(mi_option_show_errors, false);
-        mi_option_set_enabled(mi_option_show_stats, false);
-        mi_option_set_enabled(mi_option_verbose, false);
-        mi_register_output(Some(syd_noop_output), std::ptr::null_mut());
-    }
-}
-
 #[inline]
 #[cold]
 fn cold() {}
