@@ -19,8 +19,8 @@ use std::{
         fd::RawFd,
         unix::ffi::{OsStrExt, OsStringExt},
     },
-    path::{Path, PathBuf},
-    sync::LazyLock,
+    path::{Component, Path, PathBuf},
+    sync::{Arc, LazyLock},
 };
 
 use btoi::btoi;
@@ -31,14 +31,19 @@ use memchr::{
 };
 use nix::{
     errno::Errno,
-    fcntl::{openat2, OFlag, OpenHow, ResolveFlag, AT_FDCWD},
+    fcntl::{OFlag, AT_FDCWD},
     libc::pid_t,
     unistd::Pid,
     NixPath,
 };
+use tinyvec::TinyVec;
 
 use crate::{
-    config::MAGIC_PREFIX, fs::tgkill, log::log_untrusted_buf, lookup::FileType,
+    compat::{openat2, OpenHow, ResolveFlag},
+    config::MAGIC_PREFIX,
+    fs::tgkill,
+    log::log_untrusted_buf,
+    lookup::FileType,
     retry::retry_on_eintr,
 };
 
@@ -56,6 +61,9 @@ pub const PATH_MAX: usize = 4096;
 /// A safe default size to use for paths.
 pub const PATH_MIN: usize = 64;
 
+/// Default capacity of the internal `TinyVec` of `XPathBuf`.
+pub const PATH_CAP: usize = 400;
+
 // This pointer is confined by seccomp for use with openat(2) for getdir_long().
 static DOTDOT: LazyLock<u64> = LazyLock::new(|| c"..".as_ptr() as *const libc::c_char as u64);
 
@@ -68,11 +76,11 @@ pub(crate) fn dotdot_with_nul() -> u64 {
 // SAFETY: k1 == k2 -> hash(k1) == hash(k2) always holds for our PartialEq impl.
 #[allow(clippy::derived_hash_with_manual_eq)]
 #[derive(Clone, Hash, Ord, PartialOrd)]
-pub struct XPathBuf(Vec<u8>);
+pub struct XPathBuf(pub(crate) TinyVec<[u8; PATH_CAP]>);
 
 impl Default for XPathBuf {
     fn default() -> Self {
-        Self(Vec::with_capacity(PATH_MIN))
+        Self(TinyVec::with_capacity(PATH_CAP))
     }
 }
 
@@ -99,14 +107,18 @@ impl PartialEq<XPathBuf> for XPath {
 impl Deref for XPathBuf {
     type Target = XPath;
 
-    #[inline]
     fn deref(&self) -> &XPath {
         XPath::from_bytes(&self.0)
     }
 }
 
 impl Borrow<XPath> for XPathBuf {
-    #[inline]
+    fn borrow(&self) -> &XPath {
+        self.deref()
+    }
+}
+
+impl Borrow<XPath> for Arc<XPathBuf> {
     fn borrow(&self) -> &XPath {
         self.deref()
     }
@@ -173,46 +185,71 @@ impl From<&OsStr> for XPathBuf {
 
 impl From<OsString> for XPathBuf {
     fn from(os: OsString) -> Self {
-        Self(os.into_vec())
+        if os.as_bytes().len() <= PATH_CAP {
+            os.as_bytes().into()
+        } else {
+            Self(TinyVec::Heap(os.into_vec()))
+        }
     }
 }
 
 impl From<String> for XPathBuf {
     fn from(s: String) -> Self {
-        Self(s.as_bytes().into())
+        if s.len() <= PATH_CAP {
+            s.as_bytes().into()
+        } else {
+            Self(TinyVec::Heap(s.into_bytes()))
+        }
     }
 }
 
 impl From<&str> for XPathBuf {
     fn from(s: &str) -> Self {
-        Self(s.as_bytes().into())
+        let mut tv = TinyVec::new();
+        tv.extend_from_slice(s.as_bytes());
+        Self(tv)
     }
 }
 
 impl From<Cow<'_, str>> for XPathBuf {
     fn from(cow: Cow<'_, str>) -> Self {
+        if cow.len() <= PATH_CAP {
+            return cow.as_bytes().into();
+        }
+
         match cow {
-            Cow::Borrowed(s) => Self(s.as_bytes().to_vec()),
-            Cow::Owned(s) => Self(s.into_bytes()),
+            Cow::Borrowed(s) => Self::from(s),
+            Cow::Owned(s) => Self::from(s),
         }
     }
 }
 
 impl From<&[u8]> for XPathBuf {
     fn from(bytes: &[u8]) -> Self {
-        Self(bytes.to_vec())
+        let mut tv = TinyVec::new();
+        tv.extend_from_slice(bytes);
+        Self(tv)
     }
 }
 
 impl From<Vec<u8>> for XPathBuf {
     fn from(vec: Vec<u8>) -> Self {
-        Self(vec)
+        if vec.len() <= PATH_CAP {
+            vec.as_slice().into()
+        } else {
+            Self(TinyVec::Heap(vec))
+        }
     }
 }
 
 impl From<VecDeque<u8>> for XPathBuf {
-    fn from(vec: VecDeque<u8>) -> Self {
-        Self(vec.into())
+    fn from(mut vec: VecDeque<u8>) -> Self {
+        if vec.len() <= PATH_CAP {
+            vec.make_contiguous();
+            vec.as_slices().0.into()
+        } else {
+            Self(TinyVec::Heap(Vec::from(vec)))
+        }
     }
 }
 
@@ -340,7 +377,6 @@ impl XPath {
     ///
     /// Note, returning error here denies access,
     /// regardless of the state of sandboxing.
-    #[inline(always)]
     pub fn check(
         &self,
         pid: Pid,
@@ -368,10 +404,11 @@ impl XPath {
         // Note, we skip checking procfs so we don't prevent pipe/socket access
         // unintentionally. Similarly we skip checking memory fd names which do not
         // represent actual file paths.
+        // As of version 3.48.0, we return EILSEQ rather than EINVAL which matches ZFS.
         let is_mfd = matches!(file_type, Some(FileType::Mfd));
         let is_proc_dir = self.starts_with(b"/proc");
-        if safe_name && !is_mfd && !is_proc_dir && self.check_name().is_err() {
-            return Err(Errno::EINVAL);
+        if safe_name && !is_mfd && !is_proc_dir {
+            self.check_name()?;
         }
         // END OF RESTRICTION 2
 
@@ -475,11 +512,11 @@ impl XPath {
     /// # Returns
     ///
     /// * `Ok(())` if the filename is valid and safe.
-    /// * `Err(Errno::EINVAL)` if the filename is invalid or unsafe.
+    /// * `Err(Errno::EILSEQ)` if the filename is invalid or unsafe.
     ///
     /// # Errors
     ///
-    /// Returns `Err(Errno::EINVAL)` if any of the validation rules are not met.
+    /// Returns `Err(Errno::EILSEQ)` if any of the validation rules are not met.
     ///
     /// # Security
     ///
@@ -493,11 +530,11 @@ impl XPath {
         let len = name.len();
 
         if len == 0 {
-            return Err(Errno::EINVAL);
+            return Err(Errno::EILSEQ);
         }
 
         // Check if the filename is valid UTF-8.
-        let name_utf8 = std::str::from_utf8(name).or(Err(Errno::EINVAL))?;
+        let name_utf8 = std::str::from_utf8(name).or(Err(Errno::EILSEQ))?;
 
         // Check if first and last character is not whitespace.
         // This includes UTF-8 whitespace.
@@ -507,7 +544,7 @@ impl XPath {
             .map(|c| c.is_whitespace())
             .unwrap_or(false)
         {
-            return Err(Errno::EINVAL);
+            return Err(Errno::EILSEQ);
         }
         if name_utf8
             .chars()
@@ -515,7 +552,7 @@ impl XPath {
             .map(|c| c.is_whitespace())
             .unwrap_or(false)
         {
-            return Err(Errno::EINVAL);
+            return Err(Errno::EILSEQ);
         }
 
         let first_byte = name[0];
@@ -523,7 +560,7 @@ impl XPath {
 
         // Check the first byte.
         if !is_permitted_initial(first_byte) {
-            return Err(Errno::EINVAL);
+            return Err(Errno::EILSEQ);
         }
 
         // Check the middle bytes (if any).
@@ -532,13 +569,13 @@ impl XPath {
                 // Only one middle byte to check.
                 let middle_byte = name[1];
                 if !is_permitted_middle(middle_byte) {
-                    return Err(Errno::EINVAL);
+                    return Err(Errno::EILSEQ);
                 }
             }
             n if n > 2 => {
                 for &b in &name[1..len - 1] {
                     if !is_permitted_middle(b) {
-                        return Err(Errno::EINVAL);
+                        return Err(Errno::EILSEQ);
                     }
                 }
             }
@@ -547,12 +584,41 @@ impl XPath {
 
         // Check the last byte.
         if !is_permitted_final(last_byte) {
-            return Err(Errno::EINVAL);
+            return Err(Errno::EILSEQ);
         }
 
         Ok(())
     }
 
+    /// Convert /proc/${pid} to /proc/self as necessary.
+    ///
+    /// This must never used in syscalls only in access check.
+    pub fn replace_proc_self<'a>(&'a self, pid: Pid) -> Cow<'a, Self> {
+        let p = if let Some(p) = self.split_prefix(b"/proc") {
+            p
+        } else {
+            return Cow::Borrowed(self);
+        };
+
+        let mut buf = itoa::Buffer::new();
+        let pid = buf.format(pid.as_raw());
+        let p = if let Some(p) = p.split_prefix(pid.as_bytes()) {
+            p
+        } else {
+            return Cow::Borrowed(self);
+        };
+
+        let mut pdir = XPathBuf::from("/proc/self");
+        // SAFETY: We do not use XPathBuf::push here,
+        // and allow slash in path. This is safe against
+        // path traversals because the path is guaranteed
+        // to be a canonicalized path.
+        pdir.append_byte(b'/');
+        pdir.append_bytes(p.as_bytes());
+
+        Cow::Owned(pdir)
+    }
+
     /// Returns a path that, when joined onto `base`, yields `self`.
     ///
     /// Expects normalized, canonical path.
@@ -717,6 +783,33 @@ impl XPath {
         }
     }
 
+    /// Returns a relative path by stripping the root component.
+    ///
+    /// # Safety
+    ///
+    /// This function does NOT assume `self` is normalized and performs
+    /// a small amount of normalization. Therefore this function is safe
+    /// to use with non-normalized paths.
+    pub fn strip_root(&self) -> XPathBuf {
+        let src = self.as_path();
+        let mut dst = XPathBuf::new();
+
+        for comp in src.components() {
+            if matches!(
+                comp,
+                Component::RootDir | Component::Prefix(_) | Component::CurDir
+            ) {
+                continue;
+            }
+            if !dst.is_empty() {
+                dst.append_byte(b'/');
+            }
+            dst.append_bytes(comp.as_os_str().as_bytes());
+        }
+
+        dst
+    }
+
     /// Returns a path that, when joined onto `base`, yields `self`.
     ///
     /// # Safety
@@ -726,7 +819,7 @@ impl XPath {
     /// # Errors
     ///
     /// If `base` is not a prefix of self (i.e., `starts_with` returns
-    /// `false`), returns `Err`.
+    /// `false`), returns `None`.
     pub fn strip_prefix(&self, base: &[u8]) -> Option<&Self> {
         if !self.starts_with(base) {
             return None;
@@ -792,12 +885,27 @@ impl XPath {
 
     /// Returns true if the path ends with a slash.
     pub fn ends_with_slash(&self) -> bool {
-        !self.is_rootfs() && self.last() == Some(b'/')
+        self.last() == Some(b'/') && !self.is_rootfs()
     }
 
     /// Check if path has a parent dir component, ie `..`.
     pub fn has_parent_dot(&self) -> bool {
-        self.contains(b"/..") || self.is_equal(b"..")
+        let bytes = self.as_bytes();
+
+        #[allow(clippy::arithmetic_side_effects)]
+        for index in memmem::Finder::new(b"..").find_iter(bytes) {
+            let is_dotdot = if index == 0 {
+                true
+            } else {
+                bytes[index - 1] == b'/'
+            } && (index + 2 == bytes.len() || bytes[index + 2] == b'/');
+
+            if is_dotdot {
+                return true;
+            }
+        }
+
+        false
     }
 
     /// Check if path starts with the `MAGIC_PREFIX`.
@@ -876,13 +984,19 @@ impl XPath {
 
     /// Check if path points to machine-id(5), /etc/hostid or /var/adm/hostid.
     pub fn is_machine_id(&self) -> bool {
-        const MACHINE_ID: &[&[u8]] = &[b"/etc/machine-id", b"/etc/hostid", b"/var/adm/hostid"];
+        const MACHINE_ID: &[&[u8]] = &[
+            b"/etc/machine-id",
+            b"/etc/hostid",
+            b"/var/adm/hostid",
+            b"/sys/class/dmi/id/product_uuid",
+            b"/sys/devices/virtual/dmi/id/product_uuid",
+        ];
         MACHINE_ID.iter().any(|f| self.is_equal(f))
     }
 
-    /// Check if path points to a proc_pid_status(5) file.
-    pub fn is_proc_pid_status(&self) -> bool {
-        self.is_proc_pid() && self.ends_with(b"/status")
+    /// Check if path points to `/proc/self/status` or friends.
+    pub fn is_proc_status(&self) -> bool {
+        self.starts_with(b"/proc") && self.ends_with(b"/status")
     }
 
     /// Check if path points to the `/proc/self` link.
@@ -991,6 +1105,16 @@ impl XPath {
         memchr(c, self.as_bytes()).is_some()
     }
 
+    /// Return the index of the given substring in path.
+    pub fn find(&self, sub: &[u8]) -> Option<usize> {
+        memmem::find(self.as_bytes(), sub)
+    }
+
+    /// Return the index of the given character in path.
+    pub fn find_char(&self, c: u8) -> Option<usize> {
+        memchr(c, self.as_bytes())
+    }
+
     /// Returns the first character of the path.
     /// Empty path returns None.
     pub fn first(&self) -> Option<u8> {
@@ -1036,7 +1160,6 @@ impl XPath {
     ///
     /// Use the bytes up to the first nul byte if one exists;
     /// otherwise use the whole byte slice.
-    #[inline]
     pub fn from_bytes_until_nul(slice: &[u8]) -> &XPath {
         // Find first nul byte if any.
         // Use the whole slice otherwise.
@@ -1323,8 +1446,22 @@ impl XPathBuf {
     }
 
     /// Append a path component, managing separators correctly.
+    ///
+    /// Panics if `path` is not a single component.
     pub fn push(&mut self, path: &[u8]) {
-        if path.first() == Some(&b'/') {
+        // SAFETY: Guard against path traversals, allow leading/trailing slash.
+        let path = XPath::from_bytes(path);
+        match path.find_char(b'/') {
+            None | Some(0) => {}
+            Some(n) if n == path.len().saturating_sub(1) => {}
+            _ => unreachable!("BUG: Path traversal detected for `{path}'"),
+        }
+        assert!(
+            !path.has_parent_dot(),
+            "BUG: Path traversal detected for `{path}'"
+        );
+
+        if path.first() == Some(b'/') {
             // Absolute path replaces pbuf.
             self.0.clear();
         } else if self.last().map(|c| c != b'/').unwrap_or(true) {
@@ -1332,7 +1469,7 @@ impl XPathBuf {
             self.append_byte(b'/');
         }
         // Append new path part.
-        self.append_bytes(path);
+        self.append_bytes(path.as_bytes());
     }
 
     /// Remove the last path component.
@@ -1346,7 +1483,6 @@ impl XPathBuf {
     ///
     /// 1. Path must be a normalized absolute path!
     /// 2. Path must not have a trailing slash!
-    #[inline]
     pub unsafe fn pop_unchecked(&mut self) {
         #[expect(clippy::arithmetic_side_effects)]
         if let Some(idx) = memrchr(b'/', &self.as_bytes()[1..]) {
@@ -1371,14 +1507,19 @@ impl XPathBuf {
         self.0.pop()
     }
 
+    /// Clear the internal `Vec`.
+    pub fn clear(&mut self) {
+        self.0.clear()
+    }
+
     /// Convert a `XPathBuf` to a `Vec`.
     pub fn into_vec(self) -> Vec<u8> {
-        self.0
+        self.0.to_vec()
     }
 
     /// Convert a `XPathBuf` to an `OsString`.
     pub fn into_os_string(self) -> OsString {
-        OsString::from_vec(self.0)
+        OsString::from_vec(self.0.to_vec())
     }
 
     /// Shorten the vector, keeping the first len elements and dropping
@@ -1469,25 +1610,21 @@ impl XPathBuf {
     }
 
     /// Returns a slice containing the entire path buffer.
-    #[inline]
     pub fn as_slice(&self) -> &[u8] {
         self.0.as_slice()
     }
 
     /// Returns a mutable slice containing the entire path buffer.
-    #[inline]
     pub fn as_mut_slice(&mut self) -> &mut [u8] {
         self.0.as_mut_slice()
     }
 
     /// Returns a pointer to the internal `Vec`.
-    #[inline]
     pub fn as_ptr(&self) -> *const u8 {
         self.0.as_ptr()
     }
 
     /// Returns a mutable pointer to the internal `Vec`.
-    #[inline]
     pub fn as_mut_ptr(&mut self) -> *mut u8 {
         self.0.as_mut_ptr()
     }
@@ -1498,20 +1635,9 @@ impl XPathBuf {
         self.0[idx] = val;
     }
 
-    /// Forces the length of the internal `Vec`.
-    ///
-    /// # Safety
-    ///
-    /// - `new_len` must be less than or equal to `capacity()`.
-    /// - The elements at `old_len..new_len` must be initialized.
-    #[inline]
-    pub unsafe fn set_len(&mut self, new_len: usize) {
-        self.0.set_len(new_len)
-    }
-
     /// Construct an empty `XPathBuf` with capacity pre-allocated.
     pub fn with_capacity(n: usize) -> Self {
-        Self(Vec::with_capacity(n))
+        Self(TinyVec::with_capacity(n))
     }
 
     /// Report capacity of path.
@@ -1522,42 +1648,36 @@ impl XPathBuf {
     /// Constructs a new, empty `XPathBuf`.
     ///
     /// The internal vector will not allocate until elements are pushed onto it.
-    pub const fn empty() -> Self {
+    pub fn empty() -> Self {
         Self::new()
     }
 
     /// Constructs a new, empty `XPathBuf`.
     ///
     /// The internal vector will not allocate until elements are pushed onto it.
-    pub const fn new() -> XPathBuf {
-        Self(Vec::new())
+    pub fn new() -> XPathBuf {
+        Self(TinyVec::new())
     }
 }
 
 /// Logs an untrusted Path, escaping it as hex if it contains control
 /// characters.
-#[inline]
-pub fn mask_path(path: &Path) -> String {
-    let (mask, _) = log_untrusted_buf(path.as_os_str().as_bytes());
-    mask
+pub fn mask_path<P: AsRef<Path> + ?Sized>(path: &P) -> String {
+    log_untrusted_buf(path.as_ref().as_os_str().as_bytes()).0
 }
 
-#[inline]
 fn is_permitted_initial(b: u8) -> bool {
     is_permitted_byte(b) && !matches!(b, b'-' | b' ' | b'~')
 }
 
-#[inline]
 fn is_permitted_middle(b: u8) -> bool {
     is_permitted_byte(b)
 }
 
-#[inline]
 fn is_permitted_final(b: u8) -> bool {
     is_permitted_byte(b) && b != b' '
 }
 
-#[inline]
 fn is_permitted_byte(b: u8) -> bool {
     // As of version 3.38.0 the following characters are removed:
     // 1. `:` which is used commonly across devfs and procfs.
@@ -1753,6 +1873,145 @@ mod tests {
         }
     }
 
+    #[test]
+    fn test_has_parent_dot() {
+        const TEST_CASES: &[(&[u8], bool)] = &[
+            (b"/home/user/..", true),
+            (b"/home/user/abc/..", true),
+            (b"/path/to/dir/..", true),
+            (b"/home/user/abc/xyz/..", true),
+            (b"/dir/..", true),
+            (b"/dir/..abc", false),
+            (b"/../", true),
+            (b"..", true),
+            (b"/..", true),
+            (b"/../../", true),
+            (b"/..file/..", true),
+            (b"/home/user/abc", false),
+            (b"/home/user/abc/xyz", false),
+            (b"/dir/abc/xyz/..file", false),
+            (b"abc/..file", false),
+            (b"dir/abc/..xyz", false),
+            (b"dir/abc/xyz/..xyz", false),
+            (b"/../", true),
+            (b"/abc/../", true),
+            (b"/abc/../xyz", true),
+            (b"/..", true),
+            (b"/dir/..file", false),
+            (b"/..file", false),
+            (b"abc/..xyz", false),
+            (b"file/..name", false),
+            (b"/..", true),
+            (b"", false),
+            (b".", false),
+            (b".a", false),
+            (b"..", true),
+            (b"...", false),
+            (b"/long/path/with/some/other/..component", false),
+            (b"/long/path/with/..other", false),
+            (b"/this/is/a/path/to/..", true),
+            (b"/home/abc/def/..", true),
+            (b"/dir/..abc/..", true),
+            (b"/path/to/../../..", true),
+            (b"/path/to/abc/../../../xyz", true),
+            (b"/../file", true),
+            (b"/file/..", true),
+            (b"/..test", false),
+            (b"/..test/file", false),
+            (b"/some/dir/..test/file", false),
+            (b"/path/with/..file", false),
+            (b"/home/user", false),
+            (b"/usr/local/bin", false),
+            (b"/test/dir", false),
+            (b"/..", true),
+            (b"/../../", true),
+            (b"/..file/..", true),
+            (b"../dir/abc/..", true),
+            (b"dir/..abc/xyz/..", true),
+            (b"/path/to/..test/dir/../file/..", true),
+            (b"/path/..to/../file", true),
+            (b"/..test/file", false),
+            (b"/test/dir/..file/..", true),
+            (b"/some/dir/..test/file/..", true),
+            (b"/path/with/..file", false),
+        ];
+
+        for (idx, &(path, expected)) in TEST_CASES.iter().enumerate() {
+            let path = XPath::from_bytes(path);
+            assert_eq!(
+                path.has_parent_dot(),
+                expected,
+                "FAIL: `{path}' at index:{idx} expected has_parent_dot:{expected}"
+            );
+        }
+    }
+
+    #[test]
+    fn test_strip_root() {
+        let cases = [
+            // Basic relative & empty
+            ("", ""),
+            (".", ""),
+            ("./", ""),
+            ("./.", ""),
+            ("././", ""),
+            ("foo", "foo"),
+            ("foo/bar", "foo/bar"),
+            ("foo/./bar", "foo/bar"),
+            ("./foo/bar", "foo/bar"),
+            // Leading root & redundant separators / dots
+            ("/", ""),
+            ("//", ""),
+            ("///", ""),
+            ("/.", ""),
+            ("/./", ""),
+            ("/./.", ""),
+            ("/././", ""),
+            ("/foo", "foo"),
+            ("/foo/", "foo"),
+            ("//foo", "foo"),
+            ("///foo", "foo"),
+            ("/./foo", "foo"),
+            ("/././foo", "foo"),
+            ("/foo/bar", "foo/bar"),
+            ("/foo/bar/", "foo/bar"),
+            ("//foo//bar///", "foo/bar"),
+            ("/./foo/./bar/.", "foo/bar"),
+            ("///.//././foo/bar", "foo/bar"),
+            // Dots inside segment names must be preserved
+            ("/foo.bar", "foo.bar"),
+            ("/foo..bar", "foo..bar"),
+            // Parent directory segments must be preserved (no over-normalization)
+            ("../foo", "../foo"),
+            ("../../foo", "../../foo"),
+            ("foo/../bar", "foo/../bar"),
+            ("/..", ".."),
+            ("/../", ".."),
+            ("/../foo", "../foo"),
+            ("/foo/..", "foo/.."),
+            ("/foo/../bar", "foo/../bar"),
+            ("/foo/../../bar", "foo/../../bar"),
+            ("/../../etc/passwd", "../../etc/passwd"),
+            ("/foo/../../../etc/passwd", "foo/../../../etc/passwd"),
+            // "Weird" but valid segments
+            ("/ foo", " foo"),
+            ("/\u{00A0}foo", "\u{00A0}foo"), // non-breaking space
+        ];
+
+        for &(input, expected) in &cases {
+            let path = XPath::from_bytes(input.as_bytes());
+            let stripped = path.strip_root();
+            let expected_path = XPath::from_bytes(expected.as_bytes());
+
+            assert_eq!(
+                stripped.as_xpath(),
+                expected_path,
+                "strip_root failed: input={input:?}, expected={expected:?}, got={:?}",
+                stripped
+            );
+        }
+    }
+
     #[test]
     fn test_path_check_file_type() {
         xpath!("/proc")
@@ -2101,7 +2360,7 @@ mod tests {
     }
 
     #[test]
-    fn test_path_push() {
+    fn test_path_push_simple() {
         // Pushing a relative path extends the existing path.
         let mut path = XPathBuf::from("/tmp");
         path.push(b"file.bk");
@@ -2123,6 +2382,20 @@ mod tests {
         assert_eq!(path.as_os_str().as_bytes(), b"/tmp/");
     }
 
+    #[test]
+    #[should_panic]
+    fn test_path_push_dotdot_simple() {
+        let mut path = XPathBuf::from("/tmp");
+        path.push(b"..");
+    }
+
+    #[test]
+    #[should_panic]
+    fn test_path_push_dotdot_with_trailing_slash() {
+        let mut path = XPathBuf::from("/tmp");
+        path.push(b"../");
+    }
+
     #[test]
     fn test_path_split() {
         // Test typical path without trailing slash
@@ -2398,7 +2671,7 @@ mod tests {
 
     // Helper to build a buffer.
     fn xp(bytes: &[u8]) -> XPathBuf {
-        XPathBuf(bytes.to_vec())
+        XPathBuf(TinyVec::Heap(bytes.to_vec()))
     }
 
     #[test]
@@ -2409,6 +2682,25 @@ mod tests {
         Ok(())
     }
 
+    #[test]
+    fn test_push_bytes_and_convert_to_vec() {
+        let mut base = XPathBuf::from("./");
+        assert_eq!(base.as_bytes(), b"./");
+        base.append_bytes(b"sigpipe.sock");
+        base.append_byte(0);
+        let v = base.into_vec();
+        assert_eq!(v, b"./sigpipe.sock\0");
+    }
+
+    #[test]
+    fn test_convert_from_string() {
+        let s = String::from("test/path");
+        let xb = XPathBuf::from(s);
+        assert_eq!(xb.as_bytes(), b"test/path");
+        let v = xb.into_vec();
+        assert_eq!(v, b"test/path");
+    }
+
     #[test]
     fn test_replace_prefix_equal_length_overwrite() -> Result<(), Errno> {
         let mut x = xp(b"abc:/tail");
