@@ -29,6 +29,7 @@ use std::{
     thread,
 };
 
+use dur::Duration;
 use libseccomp::ScmpFilterContext;
 use nix::{
     errno::Errno,
@@ -45,8 +46,8 @@ use crate::{
     debug,
     err::{err2no, scmp2no, SydJoinHandle, SydResult},
     error,
-    fs::{block_signal, closeexcept, seccomp_export_pfc},
-    hash::SydHashMap,
+    fd::closeexcept,
+    fs::{block_signal, seccomp_export_pfc},
     hook::HandlerMap,
     info,
     retry::{retry_on_eintr, retry_on_intr},
@@ -57,7 +58,8 @@ use crate::{
         emu::Worker,
         int::Interrupter,
         ipc::IpcWorker,
-        UnixMap, WorkerCache, WorkerData,
+        out::Timeouter,
+        WorkerCache, WorkerData,
     },
 };
 
@@ -71,11 +73,9 @@ pub(crate) struct ThreadPool {
     keep_alive: u16,
     flags: Flags,
     seccomp_fd: RawFd,
-    pub(crate) cache: Arc<WorkerCache<'static>>,
+    pub(crate) cache: Arc<WorkerCache>,
     sandbox: Arc<RwLock<Sandbox>>,
     handlers: Arc<HandlerMap>,
-    crypt_map: Option<AesLock>,
-    unix_map: UnixMap,
     should_exit: Arc<AtomicBool>,
     worker_data: Arc<WorkerData>,
 }
@@ -103,16 +103,14 @@ impl ThreadPool {
     ) -> Self {
         Self {
             sandbox,
-            crypt_map,
             handlers,
             core_size,
             keep_alive,
             flags,
             should_exit,
             seccomp_fd,
-            cache: Arc::new(WorkerCache::new()),
+            cache: Arc::new(WorkerCache::new(crypt_map)),
             worker_data: Arc::new(WorkerData::default()),
-            unix_map: Arc::new(RwLock::new(SydHashMap::default())),
         }
     }
 
@@ -125,7 +123,7 @@ impl ThreadPool {
         match ExportMode::from_env() {
             Some(ExportMode::BerkeleyPacketFilter) => {
                 // Worker rules
-                let is_crypt = self.crypt_map.is_some();
+                let is_crypt = self.cache.crypt_map.is_some();
                 let ctx = Worker::prepare_confine(self.seccomp_fd, self.flags, is_crypt, &[], &[])?;
                 let file = OpenOptions::new()
                     .write(true)
@@ -180,7 +178,7 @@ impl ThreadPool {
                     "# Syd monitor rules with seccomp fd {}",
                     self.seccomp_fd
                 )?;
-                let is_crypt = self.crypt_map.is_some();
+                let is_crypt = self.cache.crypt_map.is_some();
                 let ctx = Worker::prepare_confine(self.seccomp_fd, self.flags, is_crypt, &[], &[])?;
                 rwrite!(stdout, "{}", seccomp_export_pfc(&ctx)?)?;
 
@@ -286,7 +284,7 @@ impl ThreadPool {
 
                 // Spawn the AES thread if encryption is on.
                 let crypt_handle = if let Some((fds, tmp)) = crypt {
-                    let map = self.crypt_map.as_ref().map(Arc::clone).ok_or(Errno::ENOKEY)?;
+                    let map = self.cache.crypt_map.as_ref().map(Arc::clone).ok_or(Errno::ENOKEY)?;
                     let should_exit = Arc::clone(&self.should_exit);
                     Some(self.try_spawn_aes(
                             fds,
@@ -299,6 +297,11 @@ impl ThreadPool {
                     None
                 };
 
+                if let Some(tmout) = sandbox.tmout {
+                    // Spawn the timeouter thread which will confine itself.
+                    self.try_spawn_timeout(tmout, &sandbox.transit_uids, &sandbox.transit_gids)?;
+                }
+
                 info!("ctx": "boot", "op": "start_monitor_thread",
                     "msg": format!("started monitor thread with pool size set to {} threads and keep alive set to {} seconds",
                         self.core_size,
@@ -315,7 +318,7 @@ impl ThreadPool {
                 //    precompute it and pass it to emulator threads for fast confinement.
                 let dry_run = env::var_os(ENV_SKIP_SCMP).is_some() || ExportMode::from_env().is_some();
                 let safe_setid = self.flags.intersects(Flags::FL_ALLOW_SAFE_SETUID | Flags::FL_ALLOW_SAFE_SETGID);
-                let is_crypt = self.crypt_map.is_some();
+                let is_crypt = self.cache.crypt_map.is_some();
 
                 let mut ctx = if !dry_run {
                     let ctx = Worker::prepare_confine(
@@ -381,7 +384,7 @@ impl ThreadPool {
 
                 // Wait for grace period to give the initial
                 // core emulator thread a chance to spawn itself.
-                std::thread::sleep(MON_GRACE_TIME);
+                std::thread::sleep(MON_GRACE_TIME.into());
 
                 loop {
                     // Confine and drop filter if sandbox is locked.
@@ -420,12 +423,12 @@ impl ThreadPool {
                         Ok(Some(_)) => {
                             // We have spawned a new emulator thread,
                             // wait for one cycle before reattempting.
-                            std::thread::sleep(MON_CYCLE_TIME);
+                            std::thread::sleep(MON_CYCLE_TIME.into());
                         }
                         Ok(None) => {
                             // We have idle threads, no need to spawn a new worker.
                             // Wait for grace period before reattempting.
-                            std::thread::sleep(MON_GRACE_TIME);
+                            std::thread::sleep(MON_GRACE_TIME.into());
                         }
                         Err(errno) => {
                             alert!("ctx": "spawn",
@@ -439,7 +442,7 @@ impl ThreadPool {
                             self.signal_int();
 
                             // Wait for grace period before reattempting.
-                            std::thread::sleep(MON_GRACE_TIME);
+                            std::thread::sleep(MON_GRACE_TIME.into());
                         }
                     }
                 }
@@ -510,6 +513,48 @@ impl ThreadPool {
         Ok(handle)
     }
 
+    /// Spawn an timeout handler thread to unblock Syd syscall
+    /// handler threads when the respective sandbox process
+    /// receives a non-restarting signal.
+    pub(crate) fn try_spawn_timeout(
+        &self,
+        tmout: Duration,
+        transit_uids: &[(Uid, Uid)],
+        transit_gids: &[(Gid, Gid)],
+    ) -> SydResult<SydJoinHandle<()>> {
+        // Set up a notification pipe and wait for
+        // the timeout worker to start and unshare CLONE_F{ILE,}S.
+        let (pipe_rd, pipe_wr) = pipe2(OFlag::O_CLOEXEC)?;
+
+        let handle = retry_on_intr(|| {
+            Timeouter::new(
+                tmout,
+                self.flags,
+                transit_uids,
+                transit_gids,
+                Arc::clone(&self.should_exit),
+            )
+            .try_spawn((pipe_rd.as_raw_fd(), pipe_wr.as_raw_fd()))
+        })?;
+
+        // Wait for startup notification.
+        let mut buf = [0u8; 1];
+        match retry_on_eintr(|| read(&pipe_rd, &mut buf[..]))? {
+            0 => {
+                // timeout thread died before unshare.
+                // This should ideally never happen.
+                return Err(Errno::EIO.into());
+            }
+            1 if buf[0] == 42 => {
+                // timeout thread unshared successfully.
+                // We can go ahead and close our copies now.
+            }
+            _ => unreachable!("BUG: The meaning of life is not {:#x}!", buf[0]),
+        }
+
+        Ok(handle)
+    }
+
     /// Try to create a new encryption thread.
     pub(crate) fn try_spawn_aes(
         &self,
@@ -602,8 +647,6 @@ impl ThreadPool {
                     keep_alive,
                     Arc::clone(&self.should_exit),
                     Arc::clone(&self.worker_data),
-                    self.crypt_map.as_ref().map(Arc::clone),
-                    Arc::clone(&self.unix_map),
                 )
                 .try_spawn(ctx)
             })?,
