@@ -24,11 +24,11 @@ use std::{
 
 use bitflags::bitflags;
 use data_encoding::HEXLOWER;
-use libseccomp::{ScmpNotifResp, ScmpNotifRespFlags};
+use libseccomp::{ScmpArch, ScmpNotifResp, ScmpNotifRespFlags};
 use memchr::memchr;
 use nix::{
     errno::Errno,
-    fcntl::{OFlag, ResolveFlag},
+    fcntl::OFlag,
     sys::{
         signal::SaFlags,
         socket::UnixAddr,
@@ -42,22 +42,24 @@ use serde::{ser::SerializeMap, Serialize};
 use zeroize::Zeroizing;
 
 use crate::{
-    compat::{fstatx, timespec_tv_nsec_t, TimeSpec32, TimeSpec64, STATX_INO, XATTR_NAME_MAX},
-    config::{MMAP_MIN_ADDR, PAGE_SIZE, PROC_FILE},
-    confine::{op2name, scmp_arch_bits, ScmpNotifReq, SydArch, Sydcall, EIDRM},
-    error,
-    fs::{
-        fd_status_flags, pidfd_getfd, pidfd_open, pidfd_send_signal, process_mrelease,
-        seccomp_notify_addfd, seccomp_notify_id_valid, to_valid_fd, unix_inodes, PIDFD_THREAD,
+    cache::UnixVal,
+    compat::{
+        fstatx, seccomp_notif_addfd, timespec_tv_nsec_t, OpenHow, ResolveFlag, TimeSpec32,
+        TimeSpec64, STATX_INO, XATTR_NAME_MAX,
     },
+    config::{MMAP_MIN_ADDR, PAGE_SIZE, PROC_FILE},
+    confine::{is_valid_ptr, op2name, scmp_arch_bits, ScmpNotifReq, SydArch, Sydcall, EIDRM},
+    error,
+    fd::{fd_status_flags, pidfd_getfd, pidfd_open, pidfd_send_signal, to_valid_fd, PIDFD_THREAD},
+    fs::{process_mrelease, seccomp_notify_addfd, seccomp_notify_id_valid, unix_inodes},
     lookup::{file_type, safe_canonicalize, safe_open_msym, CanonicalPath, FileType, FsFlags},
     path::{XPath, XPathBuf, PATH_MAX, PATH_MIN},
     proc::{
-        proc_auxv, proc_comm, proc_maps, proc_rand_fd, proc_stack_pointer, proc_status, proc_tgid,
-        proc_unix_inodes,
+        proc_auxv, proc_comm, proc_get_vma, proc_rand_fd, proc_stack_pointer, proc_status,
+        proc_tgid, proc_unix_inodes,
     },
     sandbox::{Action, Flags, Sandbox, SandboxGuard},
-    workers::{aes::AesLock, UnixMap, UnixVal, WorkerCache},
+    workers::WorkerCache,
 };
 
 /*
@@ -189,10 +191,10 @@ impl SysArg {
 }
 
 // Represents path arguments (max=2).
-pub(crate) type PathArg<'a> = Option<CanonicalPath<'a>>;
+pub(crate) type PathArg = Option<CanonicalPath>;
 
 #[derive(Debug)]
-pub(crate) struct PathArgs<'a>(pub(crate) PathArg<'a>, pub(crate) PathArg<'a>);
+pub(crate) struct PathArgs(pub(crate) PathArg, pub(crate) PathArg);
 
 /// By using `RemoteProcess`, you can get information about the
 /// supervised process.
@@ -238,22 +240,16 @@ impl RemoteProcess {
     /// is what's expected.
     #[expect(clippy::cognitive_complexity)]
     #[expect(clippy::type_complexity)]
-    pub(crate) fn read_path<'b>(
+    pub(crate) fn read_path(
         &self,
         sandbox: &SandboxGuard,
+        arch: ScmpArch,
         args: [u64; 6],
         arg: SysArg,
         request: Option<&UNotifyEventRequest>,
-    ) -> Result<(CanonicalPath<'b>, bool, bool, bool), Errno> {
+    ) -> Result<(CanonicalPath, bool, bool, bool), Errno> {
         let orig = match arg.path {
-            Some(idx) => {
-                // SAFETY: Check pointer against mmap_min_addr.
-                if args[idx] < *MMAP_MIN_ADDR {
-                    return Err(Errno::EFAULT);
-                }
-
-                Some(self.remote_path(args[idx], request)?)
-            }
+            Some(idx) => Some(self.remote_path(arch, args[idx], request)?),
             None => None,
         };
         let mut doterr = false;
@@ -364,31 +360,7 @@ impl RemoteProcess {
                     None
                 };
 
-                let mut path =
-                    safe_canonicalize(self.pid, fd, &path, arg.fsflags, Some(sandbox.deref()))?;
-
-                // Convert /proc/${pid} to /proc/self as necessary.
-                // This is never used in syscalls only in access check.
-                if path.base.is_empty() {
-                    if let Some(p) = path.abs().split_prefix(b"/proc") {
-                        let mut buf = itoa::Buffer::new();
-                        let pid = buf.format(self.pid.as_raw());
-                        if let Some(p) = p.split_prefix(pid.as_bytes()) {
-                            let mut pdir = XPathBuf::from("/proc/self");
-                            // SAFETY: We do not use XPathBuf::push here,
-                            // and allow slash in path. This is safe against
-                            // path traversals because the path is guaranteed
-                            // to be a canonicalized path.
-                            pdir.append_byte(b'/');
-                            pdir.append_bytes(p.as_bytes());
-
-                            // SAFETY: path.base is checked to be empty.
-                            path.set_abs(pdir);
-                        }
-                    }
-                }
-
-                path
+                safe_canonicalize(self.pid, fd, &path, arg.fsflags, Some(sandbox.deref()))?
             }
         } else {
             // SAFETY: SysArg.path is None asserting dirfd is Some.
@@ -451,11 +423,12 @@ impl RemoteProcess {
     /// This function is unsafe because the request is not validated.
     pub(crate) unsafe fn read_vec_zeroed(
         &self,
+        arch: ScmpArch,
         remote_addr: u64,
         len: usize,
     ) -> Result<Zeroizing<Vec<u8>>, Errno> {
         if Sandbox::use_proc_pid_mem() {
-            return self.read_vec_zeroed_proc(remote_addr, len);
+            return self.read_vec_zeroed_proc(arch, remote_addr, len);
         }
 
         let mut local_buffer = Zeroizing::new(Vec::new());
@@ -467,7 +440,7 @@ impl RemoteProcess {
 
         // SAFETY: Check pointer against mmap_min_addr before allocation,
         // but after length is zero check.
-        if remote_addr < *MMAP_MIN_ADDR {
+        if !is_valid_ptr(remote_addr, arch) {
             return Err(Errno::EFAULT);
         }
         let remote_addr = usize::try_from(remote_addr).or(Err(Errno::EFAULT))?;
@@ -502,9 +475,14 @@ impl RemoteProcess {
     /// # Safety
     ///
     /// This function is unsafe because the request is not validated.
-    pub(crate) unsafe fn read_vec(&self, remote_addr: u64, len: usize) -> Result<Vec<u8>, Errno> {
+    pub(crate) unsafe fn read_vec(
+        &self,
+        arch: ScmpArch,
+        remote_addr: u64,
+        len: usize,
+    ) -> Result<Vec<u8>, Errno> {
         if Sandbox::use_proc_pid_mem() {
-            return self.read_vec_proc(remote_addr, len);
+            return self.read_vec_proc(arch, remote_addr, len);
         }
 
         let mut local_buffer = Vec::new();
@@ -516,7 +494,7 @@ impl RemoteProcess {
 
         // SAFETY: Check pointer against mmap_min_addr before allocation,
         // but after length is zero check.
-        if remote_addr < *MMAP_MIN_ADDR {
+        if !is_valid_ptr(remote_addr, arch) {
             return Err(Errno::EFAULT);
         }
         let remote_addr = usize::try_from(remote_addr).or(Err(Errno::EFAULT))?;
@@ -553,16 +531,17 @@ impl RemoteProcess {
     /// This function is unsafe because the request is not validated.
     pub(crate) unsafe fn read_mem(
         &self,
+        arch: ScmpArch,
         local_buffer: &mut [u8],
         remote_addr: u64,
         len: usize,
     ) -> Result<usize, Errno> {
         if Sandbox::use_proc_pid_mem() {
-            return self.read_mem_proc(local_buffer, remote_addr, len);
+            return self.read_mem_proc(arch, local_buffer, remote_addr, len);
         }
 
         // SAFETY: Check pointer against mmap_min_addr.
-        if remote_addr < *MMAP_MIN_ADDR {
+        if !is_valid_ptr(remote_addr, arch) {
             return Err(Errno::EFAULT);
         }
         let remote_addr = usize::try_from(remote_addr).or(Err(Errno::EFAULT))?;
@@ -584,6 +563,7 @@ impl RemoteProcess {
     /// This function is unsafe because the request is not validated.
     pub(crate) unsafe fn read_vec_zeroed_proc(
         &self,
+        arch: ScmpArch,
         remote_addr: u64,
         len: usize,
     ) -> Result<Zeroizing<Vec<u8>>, Errno> {
@@ -596,7 +576,7 @@ impl RemoteProcess {
 
         // SAFETY: Check pointer against mmap_min_addr before allocation,
         // but after length is zero check.
-        if remote_addr < *MMAP_MIN_ADDR {
+        if !is_valid_ptr(remote_addr, arch) {
             return Err(Errno::EFAULT);
         }
 
@@ -644,6 +624,7 @@ impl RemoteProcess {
     /// This function is unsafe because the request is not validated.
     pub(crate) unsafe fn read_vec_proc(
         &self,
+        arch: ScmpArch,
         remote_addr: u64,
         len: usize,
     ) -> Result<Vec<u8>, Errno> {
@@ -656,7 +637,7 @@ impl RemoteProcess {
 
         // SAFETY: Check pointer against mmap_min_addr before allocation,
         // but after length is zero check.
-        if remote_addr < *MMAP_MIN_ADDR {
+        if !is_valid_ptr(remote_addr, arch) {
             return Err(Errno::EFAULT);
         }
 
@@ -704,12 +685,13 @@ impl RemoteProcess {
     /// This function is unsafe because the request is not validated.
     pub(crate) unsafe fn read_mem_proc(
         &self,
+        arch: ScmpArch,
         local_buffer: &mut [u8],
         remote_addr: u64,
         len: usize,
     ) -> Result<usize, Errno> {
         // SAFETY: Check pointer against mmap_min_addr.
-        if remote_addr < *MMAP_MIN_ADDR {
+        if !is_valid_ptr(remote_addr, arch) {
             return Err(Errno::EFAULT);
         }
 
@@ -744,15 +726,16 @@ impl RemoteProcess {
     /// This function is unsafe because the request is not validated.
     pub(crate) unsafe fn write_mem(
         &self,
+        arch: ScmpArch,
         local_buffer: &[u8],
         remote_addr: u64,
     ) -> Result<usize, Errno> {
         if Sandbox::use_proc_pid_mem() {
-            return self.write_mem_proc(local_buffer, remote_addr);
+            return self.write_mem_proc(arch, local_buffer, remote_addr);
         }
 
         // SAFETY: Check pointer against mmap_min_addr.
-        if remote_addr < *MMAP_MIN_ADDR {
+        if !is_valid_ptr(remote_addr, arch) {
             return Err(Errno::EFAULT);
         }
 
@@ -777,11 +760,12 @@ impl RemoteProcess {
     /// This function is unsafe because the request is not validated.
     pub(crate) unsafe fn write_mem_proc(
         &self,
+        arch: ScmpArch,
         local_buffer: &[u8],
         remote_addr: u64,
     ) -> Result<usize, Errno> {
         // SAFETY: Check pointer against mmap_min_addr.
-        if remote_addr < *MMAP_MIN_ADDR {
+        if !is_valid_ptr(remote_addr, arch) {
             return Err(Errno::EFAULT);
         } else if local_buffer.is_empty() {
             return Ok(0);
@@ -820,6 +804,7 @@ impl RemoteProcess {
     /// is what's expected.
     pub(crate) fn remote_path(
         &self,
+        arch: ScmpArch,
         addr: u64,
         request: Option<&UNotifyEventRequest>,
     ) -> Result<XPathBuf, Errno> {
@@ -839,7 +824,7 @@ impl RemoteProcess {
             // SAFETY:
             // 1. Assume error on zero-read.
             // 2. Validate the request after memory read.
-            let len = unsafe { self.read_mem(ptr, addr + off as u64, len) }?;
+            let len = unsafe { self.read_mem(arch, ptr, addr + off as u64, len) }?;
             if len == 0 {
                 return Err(Errno::EFAULT);
             }
@@ -876,10 +861,8 @@ pub(crate) struct UNotifyEventRequest {
     pub(crate) scmpreq: ScmpNotifReq,
     pub(crate) syscall: Sydcall,
     notify_fd: RawFd,
-    pub(crate) cache: Arc<WorkerCache<'static>>,
+    pub(crate) cache: Arc<WorkerCache>,
     sandbox: Arc<RwLock<Sandbox>>,
-    pub(crate) crypt_map: Option<AesLock>,
-    pub(crate) unix_map: UnixMap,
 }
 
 impl Serialize for UNotifyEventRequest {
@@ -893,7 +876,7 @@ impl Serialize for UNotifyEventRequest {
         map.serialize_entry("pid", &self.scmpreq.pid)?;
         map.serialize_entry("sys", &self.syscall)?;
         map.serialize_entry("args", &self.scmpreq.data.args)?;
-        map.serialize_entry("arch", &SydArch(self.scmpreq.data.arch))?;
+        map.serialize_entry("arch", &SydArch::from(self.scmpreq.data.arch))?;
 
         let pid = self.scmpreq.pid();
         if let Ok(comm) = proc_comm(pid) {
@@ -960,38 +943,33 @@ impl Serialize for UNotifyEventRequest {
             }
         }
 
-        let sp = proc_stack_pointer(pid)
-            .map(|sp| (sp & !0xF).saturating_sub(16))
-            .ok();
+        let ip = self.scmpreq.data.instr_pointer;
+        let sp = proc_stack_pointer(pid).ok();
+        map.serialize_entry("ip", &ip)?;
         map.serialize_entry("sp", &sp)?;
 
-        let ip = self.scmpreq.data.instr_pointer;
-        map.serialize_entry("ip", &ip)?;
+        let ip_vma = proc_get_vma(pid, ip).ok();
+        let sp_vma = sp.and_then(|sp| proc_get_vma(pid, sp).ok());
+        map.serialize_entry("ip_vma", &ip_vma)?;
+        map.serialize_entry("sp_vma", &sp_vma)?;
 
         let mut ip_mem = [0u8; 64];
         let mut sp_mem = [0u8; 64];
         let mut ip_read = false;
         let mut sp_read = false;
 
-        if ip >= *MMAP_MIN_ADDR && self.read_mem(&mut ip_mem, ip, 64).is_ok() {
+        if is_valid_ptr(ip, self.scmpreq.data.arch) && self.read_mem(&mut ip_mem, ip, 64).is_ok() {
             ip_read = true;
         }
 
         if let Some(sp) = sp {
-            if sp >= *MMAP_MIN_ADDR && self.read_mem(&mut sp_mem, sp, 64).is_ok() {
+            if is_valid_ptr(ip, self.scmpreq.data.arch)
+                && self.read_mem(&mut sp_mem, sp, 64).is_ok()
+            {
                 sp_read = true;
             }
         }
 
-        map.serialize_entry(
-            "sp_mem",
-            &if sp_read {
-                Some(HEXLOWER.encode(&sp_mem))
-            } else {
-                None
-            },
-        )?;
-
         map.serialize_entry(
             "ip_mem",
             &if ip_read {
@@ -1001,7 +979,14 @@ impl Serialize for UNotifyEventRequest {
             },
         )?;
 
-        map.serialize_entry("maps", &proc_maps(pid).ok())?;
+        map.serialize_entry(
+            "sp_mem",
+            &if sp_read {
+                Some(HEXLOWER.encode(&sp_mem))
+            } else {
+                None
+            },
+        )?;
 
         map.end()
     }
@@ -1012,10 +997,8 @@ impl UNotifyEventRequest {
         scmpreq: ScmpNotifReq,
         syscall: Sydcall,
         notify_fd: RawFd,
-        cache: Arc<WorkerCache<'static>>,
+        cache: Arc<WorkerCache>,
         sandbox: Arc<RwLock<Sandbox>>,
-        crypt_map: Option<AesLock>,
-        unix_map: UnixMap,
     ) -> Self {
         UNotifyEventRequest {
             scmpreq,
@@ -1023,8 +1006,6 @@ impl UNotifyEventRequest {
             notify_fd,
             cache,
             sandbox,
-            crypt_map,
-            unix_map,
         }
     }
 
@@ -1094,7 +1075,11 @@ impl UNotifyEventRequest {
     /// Used for informational purposes at recvfrom(2) boundary.
     pub(crate) fn find_unix_addr(&self, base: &XPath) -> Result<UnixAddr, Errno> {
         self.clr_unix()?; // cleanup bind-map from unused inodes.
-        let unix_map = self.unix_map.read().unwrap_or_else(|err| err.into_inner());
+        let unix_map = self
+            .cache
+            .unix_map
+            .read()
+            .unwrap_or_else(|err| err.into_inner());
         for unix_val in unix_map.values() {
             if let Some(addr) = unix_val.addr {
                 if let Some(path) = addr.path() {
@@ -1109,6 +1094,26 @@ impl UNotifyEventRequest {
         Err(Errno::ENOENT)
     }
 
+    /// Add a ptrace(PTRACE_TRACEME) attempt to the PtraceMap.
+    ///
+    /// Returns `Err(Errno::EPERM)` if the tid has already tried before.
+    pub(crate) fn add_ptrace(&self, tid: Pid) -> Result<(), Errno> {
+        let mut ptrace_map = self
+            .cache
+            .ptrace_map
+            .write()
+            .unwrap_or_else(|err| err.into_inner());
+
+        if ptrace_map.contains_key(&tid) {
+            return Err(Errno::EPERM);
+        }
+
+        let tgid = proc_tgid(tid)?;
+        ptrace_map.insert(tid, tgid);
+
+        Ok(())
+    }
+
     /// Add a bind address to the UnixMap. This has been split from the sandbox policy
     /// as of version 3.33.1 because it has no bearing on access rights and is provided
     /// for convenience for getpeername(2), getsockname(2), recvfrom(2), and recvmsg(2).
@@ -1128,7 +1133,11 @@ impl UNotifyEventRequest {
         let pid = proc_tgid(tid)?;
 
         // Record/merge unix address.
-        let mut unix_map = self.unix_map.write().unwrap_or_else(|err| err.into_inner());
+        let mut unix_map = self
+            .cache
+            .unix_map
+            .write()
+            .unwrap_or_else(|err| err.into_inner());
         match unix_map.entry(inode) {
             Entry::Occupied(mut entry) => {
                 let entry = entry.get_mut();
@@ -1166,13 +1175,18 @@ impl UNotifyEventRequest {
         // Try netlink(7) first, fallback to proc_net(5).
         // netlink(7) requires CONFIG_UNIX_DIAG enabled.
         let inodes = unix_inodes().or_else(|_| proc_unix_inodes(self.scmpreq.pid()))?;
-        let mut unix_map = self.unix_map.write().unwrap_or_else(|err| err.into_inner());
+        let mut unix_map = self
+            .cache
+            .unix_map
+            .write()
+            .unwrap_or_else(|err| err.into_inner());
         unix_map.retain(|inode, _| inodes.contains(inode));
         Ok(())
     }
 
     pub(crate) fn get_unix(&self, inode: u64) -> Option<UnixVal> {
-        self.unix_map
+        self.cache
+            .unix_map
             .read()
             .unwrap_or_else(|err| err.into_inner())
             .get(&inode)
@@ -1249,7 +1263,7 @@ impl UNotifyEventRequest {
             // Read remote memory.
             //
             // SAFETY: The request is going to be validated.
-            let n = unsafe { process.read_mem(slice, read_addr, slice.len()) }?;
+            let n = unsafe { process.read_mem(req.data.arch, slice, read_addr, slice.len()) }?;
 
             // SAFETY: Assume error on zero-read.
             if n == 0 {
@@ -1279,11 +1293,11 @@ impl UNotifyEventRequest {
         Ok(SaFlags::from_bits_truncate(raw))
     }
 
-    /// Read the `libc::open_how` struct from process memory
+    /// Read the `OpenHow` struct from process memory
     /// at the given address and size.
-    pub(crate) fn remote_ohow(&self, addr: u64, size: u64) -> Result<libc::open_how, Errno> {
+    pub(crate) fn remote_ohow(&self, addr: u64, size: u64) -> Result<OpenHow, Errno> {
         const OPEN_HOW_SIZE_VER0: usize = 24;
-        const OPEN_HOW_SIZE_LATEST: usize = size_of::<libc::open_how>();
+        const OPEN_HOW_SIZE_LATEST: usize = size_of::<OpenHow>();
 
         // SAFETY: Validate size argument.
         let size = usize::try_from(size).or(Err(Errno::EINVAL))?;
@@ -1295,7 +1309,7 @@ impl UNotifyEventRequest {
         }
 
         // SAFETY: Validate address argument.
-        if addr < *MMAP_MIN_ADDR {
+        if !is_valid_ptr(addr, self.scmpreq.data.arch) {
             return Err(Errno::EFAULT);
         }
 
@@ -1303,6 +1317,12 @@ impl UNotifyEventRequest {
         // Size is already capped to page size.
         let raw = self.read_vec(addr, size)?;
 
+        // SAFETY: Validate that the full size was read.
+        // Partial read means EFAULT.
+        if raw.len() != size {
+            return Err(Errno::EFAULT);
+        }
+
         // SAFETY: Verify trailing bytes are zero; otherwise E2BIG.
         if raw.iter().skip(OPEN_HOW_SIZE_LATEST).any(|&b| b != 0) {
             return Err(Errno::E2BIG);
@@ -1561,16 +1581,21 @@ impl UNotifyEventRequest {
     /// Check for magic prefix is magic is true.
     ///
     /// Returns `CanonicalPath` and two booleans is-magic and is-empty-path.
-    pub(crate) fn read_path<'b>(
+    pub(crate) fn read_path(
         &self,
         sandbox: &SandboxGuard,
         arg: SysArg,
-    ) -> Result<(CanonicalPath<'b>, bool, bool), Errno> {
+    ) -> Result<(CanonicalPath, bool, bool), Errno> {
         let process = RemoteProcess::new(self.scmpreq.pid());
 
         // SAFETY: The request is validated by read_path.
-        let (path, magic, doterr, empty_path) =
-            process.read_path(sandbox, self.scmpreq.data.args, arg, Some(self))?;
+        let (path, magic, doterr, empty_path) = process.read_path(
+            sandbox,
+            self.scmpreq.data.arch,
+            self.scmpreq.data.args,
+            arg,
+            Some(self),
+        )?;
 
         // Determine FD-only system calls.
         // We return EACCES rather than ENOENT for these.
@@ -1614,7 +1639,7 @@ impl UNotifyEventRequest {
         let process = RemoteProcess::new(self.scmpreq.pid());
 
         // SAFETY: The request is validated.
-        match unsafe { process.read_vec_zeroed(remote_addr, len) } {
+        match unsafe { process.read_vec_zeroed(self.scmpreq.data.arch, remote_addr, len) } {
             Ok(vec) => {
                 if self.is_valid() {
                     Ok(vec)
@@ -1631,7 +1656,7 @@ impl UNotifyEventRequest {
         let process = RemoteProcess::new(self.scmpreq.pid());
 
         // SAFETY: The request is validated.
-        match unsafe { process.read_vec(remote_addr, len) } {
+        match unsafe { process.read_vec(self.scmpreq.data.arch, remote_addr, len) } {
             Ok(vec) => {
                 if self.is_valid() {
                     Ok(vec)
@@ -1653,7 +1678,7 @@ impl UNotifyEventRequest {
         let process = RemoteProcess::new(self.scmpreq.pid());
 
         // SAFETY: The request is validated.
-        match unsafe { process.read_mem(local_buffer, remote_addr, len) } {
+        match unsafe { process.read_mem(self.scmpreq.data.arch, local_buffer, remote_addr, len) } {
             Ok(n) => {
                 if self.is_valid() {
                     Ok(n)
@@ -1674,7 +1699,7 @@ impl UNotifyEventRequest {
         let process = RemoteProcess::new(self.scmpreq.pid());
 
         // SAFETY: The request is validated.
-        match unsafe { process.write_mem(local_buffer, remote_addr) } {
+        match unsafe { process.write_mem(self.scmpreq.data.arch, local_buffer, remote_addr) } {
             Ok(n) => {
                 if self.is_valid() {
                     Ok(n)
@@ -1799,7 +1824,7 @@ impl UNotifyEventRequest {
         };
 
         #[expect(clippy::cast_sign_loss)]
-        let addfd: libc::seccomp_notif_addfd = libc::seccomp_notif_addfd {
+        let addfd: seccomp_notif_addfd = seccomp_notif_addfd {
             id: self.scmpreq.id,
             srcfd: src_fd.as_fd().as_raw_fd() as u32,
             newfd: newfd as u32,
@@ -1839,7 +1864,7 @@ impl UNotifyEventRequest {
         };
 
         #[expect(clippy::cast_sign_loss)]
-        let addfd: libc::seccomp_notif_addfd = libc::seccomp_notif_addfd {
+        let addfd: seccomp_notif_addfd = seccomp_notif_addfd {
             id: self.scmpreq.id,
             srcfd: src_fd.as_fd().as_raw_fd() as u32,
             newfd: newfd as u32,
