@@ -58,7 +58,7 @@ use std::{
     io::{stdout, BufWriter, Write},
     net::{IpAddr, SocketAddrV4, SocketAddrV6},
     os::{
-        fd::AsRawFd,
+        fd::{AsFd, AsRawFd, BorrowedFd, IntoRawFd},
         unix::{ffi::OsStrExt, fs::OpenOptionsExt, net::UnixStream, process::CommandExt},
     },
     path::Path,
@@ -76,27 +76,26 @@ use nix::{
     pty::{grantpt, unlockpt},
     sched::{unshare, CloneFlags},
     sys::{
+        resource::Resource,
         signal::Signal,
         socket::{bind, socket, AddressFamily, SockFlag, SockType, SockaddrIn, SockaddrIn6},
         stat::{umask, Mode},
         wait::{Id, WaitPidFlag},
     },
     time::{clock_gettime, ClockId},
-    unistd::{fork, getgid, getpid, getuid, isatty, sethostname, setsid, ForkResult, Pid},
+    unistd::{getgid, getpid, getuid, isatty, sethostname, setsid, Pid},
 };
 use sendfd::SendWithFd;
 use syd::{
     caps,
-    compat::{set_name, set_pdeathsig, waitid, WaitStatus},
+    compat::{clone3, set_name, set_pdeathsig, waitid, CloneArgs, WaitStatus},
     config::*,
-    confine::{confine_scmp_madvise, ExportMode},
+    confine::{confine_rlimit_zero, confine_scmp_madvise, ExportMode},
     err::err2no,
     error,
-    fs::{
-        closeexcept, format_clone_flags, format_clone_names, openpt, openpts, pidfd_open,
-        set_cloexec,
-    },
-    hash::{get_at_random_hex, hash, HashAlgorithm, SydHashSet},
+    fd::{closeexcept, pidfd_open, set_cloexec},
+    fs::{format_clone_flags, format_clone_names, openpt, openpts},
+    hash::{get_at_random_hex, hash, HashAlgorithm},
     hook::Supervisor,
     ignore_signals, info,
     landlock_policy::LandlockPolicy,
@@ -104,6 +103,7 @@ use syd::{
     loopback_set_up,
     path::XPathBuf,
     proc::proc_write,
+    rng::duprand,
     sandbox::Sandbox,
     seal::ensure_sealed,
     set_sigpipe_dfl, syd_code_name, syd_info,
@@ -320,7 +320,6 @@ syd::main! {
     let mut sandbox: Sandbox = Sandbox::default();
     let mut cmd_arg0: Option<OsString> = None;
     let mut cmd_argv: Vec<OsString> = vec![];
-    let mut cmd_envp: SydHashSet<OsString> = SydHashSet::default();
 
     // SYD_PROXY_{HOST,PORT,UNIX} -> proxy/ext/{host,port,unix}
     #[expect(clippy::disallowed_methods)]
@@ -489,7 +488,7 @@ syd::main! {
                 let value = parser.value()?.parse::<String>()?;
                 match value.split_once('=') {
                     Some((var, val)) => {
-                        cmd_envp.insert(OsString::from(var));
+                        sandbox.env_add_pass(var)?;
                         if !val.is_empty() {
                             // This way we give the user the chance to pass-through
                             // denylisted environment variables e.g.
@@ -500,7 +499,7 @@ syd::main! {
                         }
                     }
                     None => {
-                        cmd_envp.remove(&OsString::from(value.clone()));
+                        sandbox.env_del_pass(&value)?;
                         env::remove_var(value);
                     }
                 }
@@ -584,7 +583,7 @@ syd::main! {
     match ignore_signals(opts) {
         Ok(()) => {
             info!("ctx": "ignore_signals",
-                "opt": opts, "msg": "ignored signals");
+                "opt": opts, "msg": "ignored all signals for signal safety");
         }
         Err(errno) => {
             error!("ctx": "ignore_signals",
@@ -608,12 +607,28 @@ syd::main! {
         && isatty(std::io::stdin()).unwrap_or(false)
         && isatty(std::io::stdout()).unwrap_or(false)
     {
+        // TIP to be used in logging.
+        const TIP: &str = "put syd-pty(1) in PATH, set CARGO_BIN_EXE_syd-pty, or set sandbox/pty:off";
+
         // Create a PIDFd of this process and clear O_CLOEXEC.
-        let pidfd = pidfd_open(getpid(), libc::PIDFD_NONBLOCK)?;
+        // PIDFD_NONBLOCK is equivalent to O_NONBLOCK,
+        // we use the latter because bionic libc doesn't define former yet.
+        #[expect(clippy::cast_sign_loss)]
+        let pidfd = pidfd_open(getpid(), OFlag::O_NONBLOCK.bits() as u32)
+            .inspect_err(|errno| {
+                error!("ctx": "spawn_pty", "op": "pidfd_open",
+                    "msg": format!("syd-pty pidfd_open error: {errno}"),
+                    "tip": TIP, "err": *errno as i32);
+            })?;
         set_cloexec(&pidfd, false)?;
 
         // Open a pseudoterminal device and clear O_CLOEXEC.
-        let pty_m = openpt(OFlag::O_RDWR | OFlag::O_NOCTTY)?;
+        let pty_m = openpt(OFlag::O_RDWR | OFlag::O_NOCTTY)
+            .inspect_err(|errno| {
+                error!("ctx": "spawn_pty", "op": "openpt",
+                    "msg": format!("syd-pty openpt error: {errno}"),
+                    "tip": TIP, "err": *errno as i32);
+            })?;
         set_cloexec(&pty_m, false)?;
 
         // Grant access to PTY and unlock.
@@ -623,7 +638,12 @@ syd::main! {
         // Open peer device.
         // We are going to pass this end to the sandbox process.
         // This uses TIOCGPTPEER ioctl(2) so O_NOFOLLOW is not needed.
-        let pty_s = openpts(&pty_m, OFlag::O_RDWR | OFlag::O_NOCTTY | OFlag::O_CLOEXEC)?;
+        let pty_s = openpts(&pty_m, OFlag::O_RDWR | OFlag::O_NOCTTY | OFlag::O_CLOEXEC)
+            .inspect_err(|errno| {
+                error!("ctx": "spawn_pty", "op": "openpts",
+                    "msg": format!("syd-pty openpts error: {errno}"),
+                    "tip": TIP, "err": *errno as i32);
+            })?;
 
         // Prepare environment of the syd-pty process.
         // Filter the environment variables to only include the list below:
@@ -679,19 +699,18 @@ syd::main! {
         }) };
         if let Err(error) = cmd.spawn() {
             let errno = err2no(&error);
-            error!("ctx": "spawn_pty",
+            error!("ctx": "spawn_pty", "op": "spawn",
                 "msg": format!("syd-pty spawn error: {error}"),
-                "tip": "put syd-pty(1) in PATH, set CARGO_BIN_EXE_syd-pty, or set sandbox/pty:off",
-                "err": errno as i32);
+                "tip": TIP, "err": errno as i32);
             return Err(error.into());
         }
         drop(pidfd);
         drop(pty_m);
         if pty_debug {
-            warn!("ctx": "spawn_pty",
+            warn!("ctx": "spawn_pty", "op": "forward_tty",
                 "msg": "syd-pty is now forwarding terminal I/O");
         } else {
-            info!("ctx": "spawn_pty",
+            info!("ctx": "spawn_pty", "op": "forward_tty",
                 "msg": "syd-pty is now forwarding terminal I/O");
         }
 
@@ -705,14 +724,30 @@ syd::main! {
 
     let proxy_debug = env::var_os("SYD_TOR_DEBUG").is_some();
     let proxy = if sandbox.has_proxy() {
+        // TIP to be used in logging.
+        const TIP: &str = "put syd-tor(1) in PATH, set CARGO_BIN_EXE_syd-tor, or set sandbox/proxy:off";
+
         // sandbox/proxy:on implies unshare/net:1
         sandbox.set_unshare_net(true);
 
         // Create a PIDFd of this process.
-        let pidfd = pidfd_open(getpid(), libc::PIDFD_NONBLOCK)?;
+        // PIDFD_NONBLOCK is equivalent to O_NONBLOCK,
+        // we use the latter because bionic libc doesn't define former yet.
+        #[expect(clippy::cast_sign_loss)]
+        let pidfd = pidfd_open(getpid(), OFlag::O_NONBLOCK.bits() as u32)
+            .inspect_err(|errno| {
+                error!("ctx": "spawn_tor", "op": "pidfd_open",
+                    "msg": format!("syd-tor pidfd_open error: {errno}"),
+                    "tip": TIP, "err": *errno as i32);
+            })?;
 
         // Create a UNIX socket pair.
-        let (stream_parent, stream_child) = UnixStream::pair()?;
+        let (stream_parent, stream_child) = UnixStream::pair()
+            .inspect_err(|error| {
+                error!("ctx": "spawn_tor", "op": "socketpair",
+                    "msg": format!("syd-tor socketpair error: {error}"),
+                    "tip": TIP, "err": err2no(error) as i32);
+            })?;
 
         // Unset the CLOEXEC flags on the file descriptors.
         // PIDFds and Rust sockets are automatically CLOEXEC.
@@ -775,19 +810,18 @@ syd::main! {
         }) };
         if let Err(error) = cmd.spawn() {
             let errno = err2no(&error);
-            error!("ctx": "spawn_tor",
+            error!("ctx": "spawn_tor", "op": "spawn",
                 "msg": format!("syd-tor spawn error: {error}"),
-                "tip": "put syd-tor(1) in PATH, set CARGO_BIN_EXE_syd-tor, or set sandbox/proxy:off",
-                "err": errno as i32);
+                "tip": TIP, "err": errno as i32);
             return Err(error.into());
         }
         drop(pidfd);
         if proxy_debug {
-            warn!("ctx": "spawn_proxy",
-                "msg": format!("proxy is now forwarding external traffic to {}", sandbox.proxy_repr()));
+            warn!("ctx": "spawn_tor", "op": "forward_net",
+                "msg": format!("syd-tor is now forwarding external traffic to {}", sandbox.proxy_repr()));
         } else {
-            info!("ctx": "spawn_proxy",
-                "msg": format!("proxy is now forwarding external traffic to {}", sandbox.proxy_repr()));
+            info!("ctx": "spawn_tor", "op": "forward_net",
+                "msg": format!("syd-tor is now forwarding external traffic to {}", sandbox.proxy_repr()));
         }
 
         // Pass the other end of the socket-pair to the new namespace.
@@ -809,7 +843,6 @@ syd::main! {
             pty_child,
             &argv0,
             cmd_argv,
-            Some(&cmd_envp),
             cmd_arg0,
         ) {
             Ok(code) => Ok(ExitCode::from(code)),
@@ -974,26 +1007,33 @@ syd::main! {
         }
     }
 
-    // Set host and domain name for uts namespace.
-    // Log on errors and continue.
+    // Set host and domain name for uts namespace as necessary.
     if sandbox.flags.unshare_uts() {
-        let domainname = sandbox.domainname.as_c_str();
-        let domainsize = domainname.to_bytes().len(); // without NUL.
-        let domainname = domainname.as_ptr().cast();
+        if let Some(domainname) = sandbox.domainname.as_ref() {
+            let domainname = domainname.as_c_str();
+            let domainsize = domainname.to_bytes().len(); // without NUL.
+            let domainname = domainname.as_ptr().cast();
 
-        // SAFETY: No setdomainname(2) wrapper in nix yet.
-        if let Err(errno) = Errno::result(unsafe { setdomainname(domainname, domainsize) }) {
-            error!("ctx": "uts", "op": "setdomainname",
-                "msg": format!("set NIS/YP domain name failed: {errno}"),
-                "err": errno as i32);
+            // SAFETY: No setdomainname(2) wrapper in nix yet.
+            if let Err(errno) = Errno::result(unsafe { setdomainname(domainname, domainsize) }) {
+                error!("ctx": "uts", "op": "setdomainname",
+                    "msg": format!("set NIS/YP domain name failed: {errno}"),
+                    "tip": "configure `uts/domain:none'",
+                    "err": errno as i32);
+                return Err(errno.into());
+            }
         }
 
-        let hostname = sandbox.hostname.as_c_str();
-        let hostname = OsStr::from_bytes(hostname.to_bytes());
-        if let Err(errno) = sethostname(hostname) {
-            error!("ctx": "uts", "op": "sethostname",
-                "msg": format!("set host name failed: {errno}"),
-                "err": errno as i32);
+        if let Some(hostname) = sandbox.hostname.as_ref() {
+            let hostname = hostname.as_c_str();
+            let hostname = OsStr::from_bytes(hostname.to_bytes());
+            if let Err(errno) = sethostname(hostname) {
+                error!("ctx": "uts", "op": "sethostname",
+                    "msg": format!("set host name failed: {errno}"),
+                    "tip": "configure `uts/host:none'",
+                    "err": errno as i32);
+                return Err(errno.into());
+            }
         }
     }
 
@@ -1034,45 +1074,77 @@ syd::main! {
     // for trace/allow_unsafe_exec_speculative:1
     let ssb = sandbox.flags.allow_unsafe_exec_speculative();
 
-    // SAFETY: fork is our safest option here.
-    let child = match unsafe { fork() }? {
-        ForkResult::Child => {
-            let retval = match Supervisor::run(
-                sandbox,
-                pty_child,
-                &argv0,
-                cmd_argv,
-                Some(&cmd_envp),
-                cmd_arg0,
-            ) {
-                Ok(retval) => libc::c_int::from(retval),
-                Err(error) => {
-                    let errno = Errno::last();
-                    eprintln!("{error:?}");
-                    errno as i32
-                }
-            };
-            // SAFETY: _exit is safe to call in child.
-            unsafe { libc::_exit(retval) };
-        }
-        ForkResult::Parent { child, .. } => {
-            // Ensure we release all unneeded resources.
-            drop(cmd_arg0);
-            drop(cmd_argv);
-            drop(cmd_envp);
-            drop(pty_child);
-            drop(sandbox);
-
-            child
-        }
+    // SAFETY: Use clone3 with CLONE_PIDFD to avoid pid recycling.
+    let mut pid_fd: libc::c_int = -1;
+    let mut clone_args = CloneArgs {
+        flags: libc::CLONE_PIDFD as u64,
+        pidfd: (&raw mut pid_fd) as u64,
+        exit_signal: libc::SIGCHLD as u64,
+        ..Default::default()
     };
 
+    let child = clone3(&mut clone_args)?;
+
+    // Run the supervisor inside child process.
+    // Syd will be the first process in new namespaces.
+    if child.as_raw() == 0 {
+        let retval = match Supervisor::run(
+            sandbox,
+            pty_child,
+            &argv0,
+            cmd_argv,
+            cmd_arg0,
+        ) {
+            Ok(retval) => i32::from(retval),
+            Err(error) => {
+                let errno = Errno::last();
+                eprintln!("{error:?}");
+                errno as i32
+            }
+        };
+        std::process::exit(retval);
+    }
+
     // Only parent ends up here and `child` is child pid.
     //
-    // Unshare CLONE_FS|CLONE_FILES for isolation.
-    // Close all file descriptors but standard error.
-    unshare(CloneFlags::CLONE_FS | CloneFlags::CLONE_FILES)?;
-    closeexcept(&[libc::STDERR_FILENO as libc::c_uint])?;
+    // Ensure we release all unneeded resources.
+    drop(cmd_arg0);
+    drop(cmd_argv);
+    drop(pty_child);
+    drop(sandbox);
+
+    // SAFETY: Randomize the pid FD for hardening.
+    let pid_fd = duprand(pid_fd, OFlag::O_CLOEXEC | OFlag::O_EXCL)?;
+
+    // SAFETY: duprand returns a valid FD on success.
+    // Create a BorrowedFd and NOT and OwnedFd, because this fd will
+    // never be explicitly closed, it will be closed on exit. Moreover,
+    // attempts to close any fd will fail due to the seccomp(2) filter
+    // which will be applied before wait loop.
+    let pid_fd = unsafe { BorrowedFd::borrow_raw(pid_fd.into_raw_fd()) };
+
+    // Close all file descriptors but stderr and pidfd.
+    // Set must be sorted because pidfd may be lower than stderr.
+    #[expect(clippy::cast_sign_loss)]
+    let mut set = vec![
+        libc::STDERR_FILENO as libc::c_uint,
+        pid_fd.as_raw_fd() as libc::c_uint,
+    ];
+    set.sort_unstable();
+    closeexcept(&set)?;
+    drop(set);
+
+    // Confine resource limits in the new process:
+    // Set nfiles, nprocs, and filesize rlimits to zero.
+    // Set locks, memory lock and msgqueue rlimits to zero.
+    confine_rlimit_zero(&[
+        Resource::RLIMIT_FSIZE,
+        Resource::RLIMIT_NOFILE,
+        Resource::RLIMIT_NPROC,
+        Resource::RLIMIT_LOCKS,
+        Resource::RLIMIT_MEMLOCK,
+        Resource::RLIMIT_MSGQUEUE,
+    ])?;
 
     // SAFETY: Set up a Landlock sandbox to disallow all access.
     let abi = syd::landlock::ABI::new_current();
@@ -1114,7 +1186,7 @@ syd::main! {
     // which we do not need at all.
     // seccomp_add_architectures(&mut ctx)?;
 
-    let allow_call = [
+    const ALLOW_SYSCALLS: &[&str] = &[
         "exit",
         "exit_group",
         "waitid",
@@ -1140,7 +1212,7 @@ syd::main! {
         #[cfg(feature = "prof")]
         "gettid",
     ];
-    for name in allow_call.iter().chain(VDSO_SYSCALLS) {
+    for name in ALLOW_SYSCALLS.iter().chain(VDSO_SYSCALLS) {
         if let Ok(syscall) = ScmpSyscall::from_name(name) {
             ctx.add_rule(ScmpAction::Allow, syscall)?;
         }
@@ -1169,12 +1241,14 @@ syd::main! {
         )?;
     }
 
+    // Load the seccomp(2) filter.
     ctx.load()?;
 
+    // All done, start the wait loop.
     loop {
         #[expect(clippy::cast_possible_truncation)]
         #[expect(clippy::cast_sign_loss)]
-        break match waitid(Id::Pid(child), WaitPidFlag::WEXITED) {
+        break match waitid(Id::PIDFd(pid_fd.as_fd()), WaitPidFlag::WEXITED) {
             Ok(WaitStatus::Exited(_, code)) =>
             {
                 #[expect(clippy::cast_possible_truncation)]
