@@ -19,7 +19,7 @@ use nix::errno::Errno;
 use crate::{
     config::*,
     cookie::safe_memfd_create,
-    fs::seal_memfd_all,
+    fd::seal_memfd_all,
     rng::{fillrandom, randint},
 };
 
@@ -236,14 +236,16 @@ pub struct RandTimer {
 
 impl RandTimer {
     /// Generates a new timer with random offsets for uptime and idle.
-    pub fn new() -> Result<Self, Errno> {
+    ///
+    /// If `timens` is `true`, offsets are zeroed.
+    pub fn new(timens: bool) -> Result<Self, Errno> {
         // Fill both buffers with cryptographically secure bytes.
         // Use a plausible max offset ~194 days (0xFF_FFFF ≈ 16.7 million seconds).
         // This yields realistic yet unpredictable uptime/idle metrics.
         Ok(Self {
             start: Instant::now(),
-            uptime_offset: randint(1..=0xFF_FFFF)?,
-            idle_offset: randint(1..=0xFF_FFFF)?,
+            uptime_offset: if timens { 0 } else { randint(1..=0xFF_FFFF)? },
+            idle_offset: if timens { 0 } else { randint(1..=0xFF_FFFF)? },
         })
     }
 
@@ -301,7 +303,7 @@ mod tests {
     /// Basic creation test: ensures RandTimer can be constructed and used.
     #[test]
     fn test_basic_creation() {
-        let rt = RandTimer::new().expect("RandTimer creation failed");
+        let rt = RandTimer::new(false).expect("RandTimer creation failed");
         let _ = rt.uptime();
         let _ = rt.idle();
     }
@@ -309,7 +311,7 @@ mod tests {
     /// Verifies monotonic behavior over a short sleep for both uptime and idle.
     #[test]
     fn test_monotonic_increase() {
-        let rt = RandTimer::new().expect("RandTimer creation failed");
+        let rt = RandTimer::new(false).expect("RandTimer creation failed");
         let before_uptime = rt.uptime();
         let before_idle = rt.idle();
         thread::sleep(Duration::from_millis(10));
@@ -332,7 +334,7 @@ mod tests {
     /// Fires many calls to `uptime` and `idle` in quick succession.
     #[test]
     fn test_rapid_fire() {
-        let rt = RandTimer::new().expect("RandTimer creation failed");
+        let rt = RandTimer::new(false).expect("RandTimer creation failed");
         for _ in 0..10_000 {
             let _ = rt.uptime();
             let _ = rt.idle();
@@ -343,7 +345,7 @@ mod tests {
     #[test]
     fn test_repeated_creation() {
         for _ in 0..1000 {
-            let rt = RandTimer::new().expect("RandTimer creation failed");
+            let rt = RandTimer::new(false).expect("RandTimer creation failed");
             assert_ne!(
                 rt.uptime(),
                 0,
@@ -366,7 +368,7 @@ mod tests {
         for _ in 0..threads {
             handles.push(thread::spawn(move || {
                 for _ in 0..iterations {
-                    let rt = RandTimer::new().unwrap();
+                    let rt = RandTimer::new(false).unwrap();
                     let _ = rt.uptime();
                     let _ = rt.idle();
                 }
@@ -383,7 +385,7 @@ mod tests {
         let iterations = 30;
         let mut offsets = Vec::new();
         for _ in 0..iterations {
-            let rt = RandTimer::new().expect("RandTimer creation failed");
+            let rt = RandTimer::new(false).expect("RandTimer creation failed");
             // "Peek" uptime offset by subtracting the measured elapsed from `uptime`.
             let elapsed = rt.start.elapsed().as_secs();
             let offset_guess = rt.uptime().wrapping_sub(elapsed);
@@ -403,7 +405,7 @@ mod tests {
         let iterations = 30;
         let mut offsets = Vec::new();
         for _ in 0..iterations {
-            let rt = RandTimer::new().expect("RandTimer creation failed");
+            let rt = RandTimer::new(false).expect("RandTimer creation failed");
             let elapsed = rt.start.elapsed().as_secs();
             let offset_guess = rt.idle().wrapping_sub(elapsed);
             offsets.push(offset_guess);
@@ -415,7 +417,7 @@ mod tests {
     /// Test artificially forcing uptime offset near u64::MAX to see if wrapping works.
     #[test]
     fn test_uptime_wrapping() {
-        let mut rt = RandTimer::new().expect("RandTimer creation failed");
+        let mut rt = RandTimer::new(false).expect("RandTimer creation failed");
         rt.uptime_offset = u64::MAX - 1;
         let before = rt.uptime();
         thread::sleep(Duration::from_secs(1));
@@ -429,7 +431,7 @@ mod tests {
     /// Test artificially forcing idle offset near u64::MAX to see if wrapping works.
     #[test]
     fn test_idle_wrapping() {
-        let mut rt = RandTimer::new().expect("RandTimer creation failed");
+        let mut rt = RandTimer::new(false).expect("RandTimer creation failed");
         rt.idle_offset = u64::MAX - 1;
         let before = rt.idle();
         thread::sleep(Duration::from_secs(1));
@@ -443,7 +445,7 @@ mod tests {
     /// Check forcibly set offsets to zero for near "raw monotonic" behavior.
     #[test]
     fn test_force_offsets_zero() {
-        let mut rt = RandTimer::new().expect("RandTimer creation failed");
+        let mut rt = RandTimer::new(false).expect("RandTimer creation failed");
         rt.uptime_offset = 0;
         rt.idle_offset = 0;
         let t1_up = rt.uptime();
@@ -468,7 +470,7 @@ mod tests {
     /// Force a very large idle offset and a small sleep. Ensures no panic or freeze.
     #[test]
     fn test_large_idle_offset_small_sleep() {
-        let mut rt = RandTimer::new().expect("RandTimer creation failed");
+        let mut rt = RandTimer::new(false).expect("RandTimer creation failed");
         rt.idle_offset = u64::MAX / 2;
         let before = rt.idle();
         thread::sleep(Duration::from_secs(1));
@@ -483,7 +485,7 @@ mod tests {
     #[test]
     fn test_big_loop_creation() {
         for i in 0..10_000 {
-            let rt = RandTimer::new().expect("RandTimer creation failed");
+            let rt = RandTimer::new(false).expect("RandTimer creation failed");
             if i % 1000 == 0 {
                 let _ = rt.uptime();
                 let _ = rt.idle();
@@ -501,7 +503,7 @@ mod tests {
             (0xFFFFFFFF_FFFFFFFF, 0x55555555_55555555),
         ];
         for &(u_off, i_off) in &test_offsets {
-            let mut rt = RandTimer::new().expect("RandTimer creation failed");
+            let mut rt = RandTimer::new(false).expect("RandTimer creation failed");
             rt.uptime_offset = u_off;
             rt.idle_offset = i_off;
             let up1 = rt.uptime();
