@@ -10,6 +10,22 @@ use std::process::ExitCode;
 
 use syd::confine::fork_fast;
 
+// Set global allocator to GrapheneOS allocator.
+#[cfg(all(
+    not(coverage),
+    not(feature = "prof"),
+    not(target_os = "android"),
+    target_page_size_4k,
+    target_pointer_width = "64"
+))]
+#[global_allocator]
+static GLOBAL: hardened_malloc::HardenedMalloc = hardened_malloc::HardenedMalloc;
+
+// Set global allocator to tcmalloc if profiling is enabled.
+#[cfg(feature = "prof")]
+#[global_allocator]
+static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
+
 syd::main! {
     use lexopt::prelude::*;
 
@@ -17,7 +33,7 @@ syd::main! {
 
     // Parse CLI options.
     let mut parser = lexopt::Parser::from_env();
-    #[allow(clippy::never_loop)]
+    #[expect(clippy::never_loop)]
     while let Some(arg) = parser.next()? {
         match arg {
             Short('h') => {
