@@ -15,7 +15,7 @@
 use std::{
     collections::{HashMap, HashSet},
     env,
-    ffi::{CString, OsStr, OsString},
+    ffi::{OsStr, OsString},
     fmt::Write as FmtWrite,
     fs::{self, DirBuilder},
     io::{BufReader, BufWriter, Write as IOWrite},
@@ -80,7 +80,8 @@ use oci_spec::runtime::{
 use procfs::process::Namespace;
 use serde_json::to_writer_pretty;
 use syd::{
-    compat::{waitid, WaitStatus},
+    compat::{set_name, waitid, WaitStatus},
+    config::ENV_SKIP_SCMP,
     err::{SydError, SydResult},
     hook::Supervisor,
     ignore_signals,
@@ -93,10 +94,16 @@ use syd::{
 use tabwriter::TabWriter;
 use tracing_subscriber::layer::SubscriberExt;
 
-// Set global allocator to mimalloc.
-#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+// Set global allocator to GrapheneOS allocator.
+#[cfg(all(
+    not(coverage),
+    not(feature = "prof"),
+    not(target_os = "android"),
+    target_page_size_4k,
+    target_pointer_width = "64"
+))]
 #[global_allocator]
-static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+static GLOBAL: hardened_malloc::HardenedMalloc = hardened_malloc::HardenedMalloc;
 
 // Set global allocator to tcmalloc if profiling is enabled.
 #[cfg(feature = "prof")]
@@ -133,7 +140,7 @@ impl Executor for SydExecutor {
         // libcontainer sets process name to youki:INIT.
         // Let's avoid the confusion but ignore errors,
         // because this is not really that important...
-        let _ = set_name("syd_oci");
+        let _ = set_name(c"syd_oci");
 
         // Log the Spec for debugging (requires features=log).
         syd::t!(
@@ -201,7 +208,7 @@ impl Executor for SydExecutor {
         ignore_signals(opts).or(Err(ExecutorError::CantHandle("Failed to ignore signals!")))?;
 
         // Run command under the Syd sandbox and exit with the return code.
-        Supervisor::run(sandbox, None, comm, argv, None, None)
+        Supervisor::run(sandbox, None, comm, argv, None)
             .map(i32::from)
             .map(exit)
             .map_err(|err| ExecutorError::Execution(err.into()))?
@@ -1491,7 +1498,9 @@ syd::main! {
             Some(val) => env::set_var("SYD_RUST_BACKTRACE", val),
             None => env::remove_var("SYD_RUST_BACKTRACE"),
         };
-        env::set_var("RUST_BACKTRACE", "0");
+        if env::var_os(ENV_SKIP_SCMP).is_none() {
+            env::set_var("RUST_BACKTRACE", "0");
+        }
     }
 
     // Dump is only used for syd(1) binary,
@@ -2340,15 +2349,6 @@ fn is_executable(path: &Path) -> std::result::Result<bool, std::io::Error> {
     Ok(metadata.is_file() && permissions.mode() & 0o001 != 0)
 }
 
-fn set_name(name: &str) -> std::result::Result<(), Errno> {
-    let name = CString::new(name).map_err(|_| Errno::EINVAL)?;
-
-    // SAFETY: syd's nix version does not have prctl::set_name yet.
-    let res = unsafe { nix::libc::prctl(nix::libc::PR_SET_NAME, name.as_ptr(), 0, 0, 0) };
-
-    Errno::result(res).map(drop)
-}
-
 // If None, it will generate a default path for cgroups.
 fn get_cgroup_path(cgroups_path: &Option<PathBuf>, container_id: &str) -> PathBuf {
     match cgroups_path {
