@@ -10,8 +10,7 @@
 // SPDX-License-Identifier: GPL-3.0
 
 // SAFETY: This module has (almost) been liberated from unsafe code!
-// 1. We call into sigaction to set SIGALRM handler with -t timeout which needs unsafe.
-// 2. We call into dup3_raw to duplicate fd with -d fd which needs unsafe.
+// 1. We call into sigaction(2) to set SIGALRM handler with -t timeout which needs unsafe.
 // Use deny rather than forbid so we can allow this case.
 #![deny(unsafe_code)]
 
@@ -34,13 +33,12 @@ use nix::{
         signalfd::SigSet,
         stat::Mode,
     },
-    unistd::dup3_raw,
 };
 use syd::{
-    compat::{openat2, OpenHow, ResolveFlag},
+    compat::{dup3, openat2, OpenHow, ResolveFlag},
     config::{ENV_SH, SYD_SH},
     confine::run_cmd,
-    fs::set_cloexec,
+    fd::set_cloexec,
     ofd::lock_fd,
     path::XPathBuf,
     retry::retry_on_eintr,
@@ -169,12 +167,10 @@ syd::main! {
 
     // Prepare to pass fd to the child process.
     if let Some(opt_fdset) = opt_fdset {
-        #[expect(unsafe_code)]
         if opt_fdset != fd.as_raw_fd() {
             // Atomically duplicate onto the exact fd number.
             // Note we move the old fd into the function so it's dropped on return.
-            // SAFETY: Caller must pass valid fd to avoid double-close.
-            fd = unsafe { dup3_raw(fd, opt_fdset, OFlag::O_CLOEXEC) }?;
+            fd = dup3(fd.as_raw_fd(), opt_fdset, OFlag::O_CLOEXEC.bits())?;
         }
     }
     set_cloexec(&fd, false)?;
