@@ -31,8 +31,7 @@ use nix::{
     sys::{
         epoll::{Epoll, EpollCreateFlags, EpollEvent, EpollFlags},
         mman::{mmap_anonymous, MapFlags, ProtFlags},
-        prctl::set_dumpable,
-        resource::{getrlimit, setrlimit, Resource},
+        resource::{getrlimit, Resource},
         socket::{
             listen, AddressFamily, Backlog, Shutdown, SockFlag, SockType, SockaddrLike,
             SockaddrStorage, UnixAddr,
@@ -43,9 +42,11 @@ use nix::{
 };
 use sendfd::RecvWithFd;
 use syd::{
-    compat::epoll_ctl_safe,
+    compat::{epoll_ctl_safe, set_dumpable},
     config::{PIPE_BUF, VDSO_SYSCALLS},
-    confine::{confine_mdwe, confine_scmp_madvise},
+    confine::{
+        confine_mdwe, confine_rlimit, confine_rlimit_zero, confine_scmp_madvise, CLONE_NEWTIME,
+    },
     cookie::{
         safe_connect, safe_pipe2, safe_shutdown, safe_socket, safer_accept4, ACCEPT4_COOKIE_ARG4,
         ACCEPT4_COOKIE_ARG5, CONNECT_COOKIE_ARG3, CONNECT_COOKIE_ARG4, CONNECT_COOKIE_ARG5,
@@ -55,7 +56,7 @@ use syd::{
         SYS_SHUTDOWN, SYS_SOCKET,
     },
     err::{SydError, SydResult},
-    fs::closeexcept,
+    fd::closeexcept,
     hash::SydHashMap,
     ignore_signals,
     landlock_policy::LandlockPolicy,
@@ -69,10 +70,16 @@ use syd::{
     IgnoreSignalOpts,
 };
 
-// Set global allocator to mimalloc.
-#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+// Set global allocator to GrapheneOS allocator.
+#[cfg(all(
+    not(coverage),
+    not(feature = "prof"),
+    not(target_os = "android"),
+    target_page_size_4k,
+    target_pointer_width = "64"
+))]
 #[global_allocator]
-static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+static GLOBAL: hardened_malloc::HardenedMalloc = hardened_malloc::HardenedMalloc;
 
 // Set global allocator to tcmalloc if profiling is enabled.
 #[cfg(feature = "prof")]
@@ -99,7 +106,7 @@ syd::main! {
     // Attempt to set file-max to hard limit overriding the soft limit.
     if let Ok((soft_limit, hard_limit)) = getrlimit(Resource::RLIMIT_NOFILE) {
         if soft_limit < hard_limit {
-            let _ = setrlimit(Resource::RLIMIT_NOFILE, hard_limit, hard_limit);
+            let _ = confine_rlimit(Resource::RLIMIT_NOFILE, Some(hard_limit));
         }
     }
 
@@ -129,8 +136,7 @@ syd::main! {
     // SAFETY: Randomize received socket fd for hardening.
     let proxy_rand = duprand(proxy.as_raw_fd(), OFlag::O_CLOEXEC)?;
     drop(proxy);
-    // SAFETY: duprand returns a valid FD on success.
-    let proxy = unsafe { OwnedFd::from_raw_fd(proxy_rand) };
+    let proxy = proxy_rand;
 
     // Start listening on the bound fd.
     // Note, Syd already set this fd non-blocking.
@@ -143,10 +149,7 @@ syd::main! {
     let epoll = Epoll::new(EpollCreateFlags::EPOLL_CLOEXEC)?;
 
     // SAFETY: Randomize the epoll fd for hardening.
-    let epoll_fd = duprand(epoll.0.as_raw_fd(), OFlag::O_CLOEXEC).map(|fd| {
-        // SAFETY: duprand returns a valid FD on success.
-        unsafe { OwnedFd::from_raw_fd(fd) }
-    })?;
+    let epoll_fd = duprand(epoll.0.as_raw_fd(), OFlag::O_CLOEXEC)?;
     drop(epoll);
 
     // Allocate a private memory area for the address pointer.
@@ -623,7 +626,7 @@ impl<'a> Proxy<'a> {
                 | CloneFlags::CLONE_NEWNS
                 | CloneFlags::CLONE_NEWPID
                 | CloneFlags::CLONE_NEWUTS
-                | CloneFlags::from_bits_retain(libc::CLONE_NEWTIME);
+                | CLONE_NEWTIME;
             if family == AddressFamily::Unix {
                 // UNIX domain socket, isolate NET.
                 namespaces |= CloneFlags::CLONE_NEWNET;
@@ -659,6 +662,18 @@ impl<'a> Proxy<'a> {
 
             // Set the process dumpable attribute to not-dumpable.
             let _ = set_dumpable(false);
+
+            // Set nprocs and filesize rlimits to zero.
+            // Set locks, memory lock and msgqueue rlimits to zero.
+            // Set core dump file size to zero.
+            confine_rlimit_zero(&[
+                Resource::RLIMIT_CORE,
+                Resource::RLIMIT_FSIZE,
+                Resource::RLIMIT_NPROC,
+                Resource::RLIMIT_LOCKS,
+                Resource::RLIMIT_MEMLOCK,
+                Resource::RLIMIT_MSGQUEUE,
+            ])?;
         }
 
         if self.print {
@@ -1160,11 +1175,9 @@ fn parse_options() -> SydResult<TorOpts> {
             // SAFETY: Randomize fd to make it harder to locate for an attacker.
             let fd = duprand(fd.as_raw_fd(), OFlag::O_CLOEXEC)?;
 
-            let unix: XPathBuf = fd.to_string().into();
-            let unix = UnixAddr::new(&unix)?;
-
-            // SAFETY: duprand returns valid FD on success.
-            unix_fd = Some(unsafe { OwnedFd::from_raw_fd(fd) });
+            let mut unix = itoa::Buffer::new();
+            let unix = UnixAddr::new(unix.format(fd.as_raw_fd()))?;
+            unix_fd = Some(fd);
 
             // SAFETY: We validated address is valid above,
             // so here we pack into a SockaddrStorage safely.
@@ -1184,10 +1197,6 @@ fn parse_options() -> SydResult<TorOpts> {
     // SAFETY: Randomize fd to make it harder to locate for an attacker.
     let fpid = duprand(fpid.as_raw_fd(), OFlag::O_CLOEXEC | OFlag::O_EXCL)?;
 
-    // SAFETY: Syd passes a valid PID FD to syd-tor.
-    // Any other usecase is unsupported.
-    let fpid = unsafe { OwnedFd::from_raw_fd(fpid) };
-
     // SAFETY: We will validate the FD below.
     let sock = unsafe { BorrowedFd::borrow_raw(sock) };
 
@@ -1196,11 +1205,7 @@ fn parse_options() -> SydResult<TorOpts> {
     fcntl(sock, FcntlArg::F_GETFD)?;
 
     // SAFETY: Randomize fd to make it harder to locate for an attacker.
-    let sock = duprand(sock.as_raw_fd(), OFlag::O_CLOEXEC | OFlag::O_EXCL)?;
-
-    // SAFETY: Syd passes a valid socket FD to syd-tor.
-    // Any other usecase is unsupported.
-    let sock = unsafe { UnixStream::from_raw_fd(sock) };
+    let sock = duprand(sock.as_raw_fd(), OFlag::O_CLOEXEC | OFlag::O_EXCL).map(UnixStream::from)?;
 
     Ok(TorOpts {
         fpid,
