@@ -47,7 +47,7 @@ use crate::{
     debug,
     err::{err2no, SydJoinHandle, SydResult},
     error,
-    fs::seal_memfd,
+    fd::seal_memfd_all,
     hash::{
         aes_ctr_enc, aes_ctr_init, hmac_sha256_feed, hmac_sha256_fini, hmac_sha256_init,
         SydHashMap, BLOCK_SIZE, HMAC_TAG_SIZE, IV, IV_SIZE, SYD3_HDR_SIZE,
@@ -635,7 +635,7 @@ impl AesWorker {
     ) -> SydResult<()> {
         // Seal memfd to ensure no further writes happen.
         if memfd {
-            seal_memfd(&crypt_fd)?;
+            seal_memfd_all(&crypt_fd)?;
             debug!("ctx": "aes", "op": "seal_memfd",
                 "path": crypt_path, "memfd": memfd,
                 "msg": "sealed plaintext memfd to prevent seals, writes, shrinks and grows");
