@@ -29,7 +29,7 @@ use nix::{
     errno::Errno,
     fcntl::OFlag,
     sched::{unshare, CloneFlags},
-    unistd::{Gid, Uid},
+    unistd::{Gid, Pid, Uid},
 };
 
 #[cfg(target_arch = "x86")]
@@ -54,7 +54,8 @@ use crate::{
         LINKAT_COOKIE_ARG5, MEMFD_CREATE_COOKIE_ARG2, MEMFD_CREATE_COOKIE_ARG3,
         MEMFD_CREATE_COOKIE_ARG4, MEMFD_CREATE_COOKIE_ARG5, OPENAT2_COOKIE_ARG4,
         OPENAT2_COOKIE_ARG5, PIPE2_COOKIE_ARG2, PIPE2_COOKIE_ARG3, PIPE2_COOKIE_ARG4,
-        PIPE2_COOKIE_ARG5, RENAMEAT2_COOKIE_ARG5, SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG3,
+        PIPE2_COOKIE_ARG5, PROCMAP_QUERY_COOKIE_ARG3, PROCMAP_QUERY_COOKIE_ARG4,
+        PROCMAP_QUERY_COOKIE_ARG5, RENAMEAT2_COOKIE_ARG5, SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG3,
         SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG4, SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG5,
         SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG3, SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG4,
         SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG5, SOCKETPAIR_COOKIE_ARG4, SOCKETPAIR_COOKIE_ARG5,
@@ -67,14 +68,15 @@ use crate::{
         UNLINKAT_COOKIE_ARG5,
     },
     err::{err2no, SydJoinHandle, SydResult},
+    fd::{close, AT_EXECVE_CHECK},
     fs::{
-        close, seccomp_notify_respond, AT_EXECVE_CHECK, SECCOMP_IOCTL_NOTIF_ADDFD,
-        SECCOMP_IOCTL_NOTIF_LIST, SECCOMP_IOCTL_NOTIF_SEND,
+        seccomp_notify_respond, SECCOMP_IOCTL_NOTIF_ADDFD, SECCOMP_IOCTL_NOTIF_LIST,
+        SECCOMP_IOCTL_NOTIF_SEND,
     },
     hook::HandlerMap,
     info,
     path::dotdot_with_nul,
-    proc::proc_mmap,
+    proc::{proc_get_vma, PROCMAP_QUERY},
     req::UNotifyEventRequest,
     sandbox::{Flags, Sandbox, SandboxGuard},
     timer::AlarmTimer,
@@ -349,9 +351,12 @@ impl Worker {
         let ghost = match response.error {
             EIDRM if response.id == 0 && response.val == 0 && response.flags == 0 => return,
             EOWNERDEAD if response.id == 0 && response.val == 0 && response.flags == 0 => {
+                #[expect(clippy::cast_possible_wrap)]
+                let pid = Pid::from_raw(req.pid as libc::pid_t);
+                let vma = proc_get_vma(pid, req.data.instr_pointer).ok();
                 crate::warn!("ctx": "confine", "op": "enter_ghost_mode", "pid": req.pid,
                     "sys": syscall, "arch": SydArch::from(req.data.arch), "args": req.data.args,
-                    "src": proc_mmap(req.pid(), req.data.instr_pointer).ok());
+                    "ip": req.data.instr_pointer, "src": vma);
 
                 // Correct the pseudo errno back to success.
                 response.error = 0;
@@ -456,15 +461,19 @@ impl Worker {
             }
         }
 
-        // Allow a restricted set of ioctl(2) operations to our seccomp fd only.
+        // Ioctl Restrictions:
+        // 1. Allow a restricted set of ioctl(2) operations to the seccomp(2) fd.
+        // 2. Allow ioctl(2) request PROCMAP_QUERY to lookup proc_pid_maps(5) efficiently.
+        //    This request is new in Linux-6.11.
         //
-        // Syscall argument cookies for SECCOMP_IOCTL_NOTIF_SEND my be disabled
-        // at startup with trace/allow_unsafe_nocookie:1.
+        // Syscall argument cookies for PROCMAP_QUERY and
+        // SECCOMP_IOCTL_NOTIF_SEND my be disabled at startup with
+        // trace/allow_unsafe_nocookie:1.
         let sysname = "ioctl";
         #[expect(clippy::cast_sign_loss)]
+        #[expect(clippy::useless_conversion)]
         match ScmpSyscall::from_name(sysname) {
             Ok(syscall) => {
-                #[expect(clippy::useless_conversion)]
                 for ioctl_request in SECCOMP_IOCTL_NOTIF_LIST {
                     let mut rules = vec![scmp_cmp!($arg0 == seccomp_fd as u64)];
 
@@ -506,6 +515,29 @@ impl Worker {
                         ctx.add_rule_conditional(ScmpAction::Allow, syscall, &rules)?;
                     }
                 }
+
+                if *HAVE_PROCMAP_QUERY {
+                    #[expect(clippy::unnecessary_cast)]
+                    let ioctl_request = PROCMAP_QUERY as u64;
+                    let mut rules =
+                        Vec::with_capacity(if flags.allow_unsafe_nocookie() { 1 } else { 4 });
+                    if !flags.allow_unsafe_nocookie() {
+                        rules.extend(&[
+                            scmp_cmp!($arg3 == (*PROCMAP_QUERY_COOKIE_ARG3).into()),
+                            scmp_cmp!($arg4 == (*PROCMAP_QUERY_COOKIE_ARG4).into()),
+                            scmp_cmp!($arg5 == (*PROCMAP_QUERY_COOKIE_ARG5).into()),
+                        ]);
+                    }
+
+                    rules.push(scmp_cmp!($arg1 == ioctl_request));
+                    ctx.add_rule_conditional(ScmpAction::Allow, syscall, &rules)?;
+
+                    if let Some(ioctl_request) = extend_ioctl(ioctl_request) {
+                        rules.pop();
+                        rules.push(scmp_cmp!($arg1 == ioctl_request));
+                        ctx.add_rule_conditional(ScmpAction::Allow, syscall, &rules)?;
+                    }
+                }
             }
             Err(_) => {
                 info!("ctx": "confine", "op": "allow_emu_syscall",
