@@ -40,7 +40,7 @@ use crate::{
         PROCMAP_QUERY_COOKIE_ARG4, PROCMAP_QUERY_COOKIE_ARG5,
     },
     err::SydResult,
-    fs::AT_EXECVE_CHECK,
+    fd::AT_EXECVE_CHECK,
     info,
     kernel::ptrace::event::{
         exec::sysevent_exec, exit::sysevent_exit, fork::sysevent_fork, scmp::sysevent_scmp,
@@ -131,7 +131,7 @@ impl Tracer {
             ) => {
                 // SAFETY: nix does not have a wrapper for PTRACE_LISTEN,s
                 // so we fallback to libc here.
-                let _ = unsafe { libc::ptrace(libc::PTRACE_LISTEN, pid.as_raw(), 0, 0) };
+                let _ = unsafe { libc::ptrace(crate::compat::PTRACE_LISTEN, pid.as_raw(), 0, 0) };
             }
             WaitStatus::PtraceEvent(
                 pid,
@@ -278,7 +278,7 @@ impl Tracer {
             ctx.add_rule_conditional(ScmpAction::Allow, syscall, &[scmp_cmp!($arg1 == 0)])?;
         }
 
-        // Allow the ioctl request PAGEMAP_QUERY to lookup _proc_pid_maps_(5) efficiently.
+        // Allow ioctl(2) request PROCMAP_QUERY to lookup proc_pid_maps(5) efficiently.
         // This request is new in Linux-6.11.
         if *HAVE_PROCMAP_QUERY {
             let sysname = "ioctl";
