@@ -13,61 +13,68 @@ use std::{
     collections::hash_map::Entry,
     fs::File,
     option::Option,
-    sync::{
-        atomic::{AtomicUsize, Ordering},
-        Arc, RwLock,
-    },
+    sync::atomic::{AtomicUsize, Ordering},
 };
 
 use nix::{
     errno::Errno,
-    sys::{
-        signal::{SigSet, Signal},
-        socket::UnixAddr,
-    },
+    sys::signal::{SigSet, Signal},
     unistd::{gettid, Pid},
 };
 
 use crate::{
     cache::{
-        signal_map_new, sys_interrupt_map_new, sys_result_map_new, ExecResult, SignalMap,
-        SysInterrupt, SysInterruptMap, SysResultMap,
+        ptrace_map_new, signal_map_new, sys_interrupt_map_new, sys_result_map_new, unix_map_new,
+        ExecResult, PtraceMap, SignalMap, SysInterrupt, SysInterruptMap, SysResultMap, UnixMap,
     },
-    confine::{ScmpNotifReq, SydMemoryMap},
+    confine::ScmpNotifReq,
     elf::ExecutableFile,
     fs::{block_signal, sigtimedpoll, unblock_signal},
-    hash::SydHashMap,
-    lookup::CanonicalPath,
     retry::retry_on_eintr,
     sigset::SydSigSet,
+    workers::aes::AesLock,
 };
 
 // syd_aes: Encryptor helper thread
 pub(crate) mod aes;
 // syd_int: Interrupter helper thread
 pub(crate) mod int;
+// syd_out: Timeouter helper thread
+pub(crate) mod out;
 // syd_ipc: IPC thread
 pub(crate) mod ipc;
 // syd_emu: Main worker threads
 pub(crate) mod emu;
+// syd_gdb: Ptrace worker thread (aka syd_main)
+pub(crate) mod gdb;
 
 /// A cache for worker threads.
-#[derive(Debug)]
-pub(crate) struct WorkerCache<'a> {
+pub(crate) struct WorkerCache {
     // Signal handlers map
     pub(crate) signal_map: SignalMap,
     // System call interrupt map
     pub(crate) sysint_map: SysInterruptMap,
     // System call result map
-    pub(crate) sysres_map: SysResultMap<'a>,
+    pub(crate) sysres_map: SysResultMap,
+    // [inode,(pid,path)] map of unix binds.
+    // Path is only used for UNIX domain sockets.
+    pub(crate) unix_map: UnixMap,
+    // [tid, tgid] map for ptrace(PTRACE_TRACEME) calling tids.
+    // This is used to prevent ptrace(2) detection efficiently.
+    pub(crate) ptrace_map: PtraceMap,
+    // Crypt sandboxing map.
+    pub(crate) crypt_map: Option<AesLock>,
 }
 
-impl<'a> WorkerCache<'a> {
-    pub(crate) fn new() -> Self {
+impl WorkerCache {
+    pub(crate) fn new(crypt_map: Option<AesLock>) -> Self {
         Self {
             signal_map: signal_map_new(),
             sysint_map: sys_interrupt_map_new(),
             sysres_map: sys_result_map_new(),
+            unix_map: unix_map_new(),
+            ptrace_map: ptrace_map_new(),
+            crypt_map,
         }
     }
 
@@ -138,22 +145,49 @@ impl<'a> WorkerCache<'a> {
         map.remove(&tgid);
     }
 
+    // Delete a TGID from ptrace map.
+    pub(crate) fn retire_ptrace(&self, tgid: Pid) {
+        let mut map = self
+            .ptrace_map
+            .write()
+            .unwrap_or_else(|err| err.into_inner());
+        map.retain(|_, &mut pid| pid != tgid)
+    }
+
     // Record a chdir result.
-    pub(crate) fn add_chdir<'b>(&'b self, pid: Pid, path: CanonicalPath<'a>) {
+    pub(crate) fn add_chdir(&self, pid: Pid) {
         self.sysres_map
             .trace_chdir
             .lock()
             .unwrap_or_else(|err| err.into_inner())
-            .insert(pid, path);
+            .insert(pid);
     }
 
     // Query, remove and return a chdir result.
-    pub(crate) fn get_chdir<'b>(&'b self, pid: Pid) -> Option<(Pid, CanonicalPath<'a>)> {
+    pub(crate) fn get_chdir(&self, pid: Pid) -> bool {
         self.sysres_map
             .trace_chdir
             .lock()
             .unwrap_or_else(|err| err.into_inner())
-            .remove_entry(&pid)
+            .remove(&pid)
+    }
+
+    // Record a mmap(2) pid.
+    pub(crate) fn add_mmap(&self, pid: Pid) {
+        self.sysres_map
+            .trace_mmap
+            .lock()
+            .unwrap_or_else(|err| err.into_inner())
+            .insert(pid);
+    }
+
+    // Query, remove and return true if found.
+    pub(crate) fn get_mmap(&self, pid: Pid) -> bool {
+        self.sysres_map
+            .trace_mmap
+            .lock()
+            .unwrap_or_else(|err| err.into_inner())
+            .remove(&pid)
     }
 
     // Record an error result.
@@ -175,29 +209,8 @@ impl<'a> WorkerCache<'a> {
     }
 
     // Record a execv result.
-    #[expect(clippy::too_many_arguments)]
-    pub(crate) fn add_exec(
-        &self,
-        pid: Pid,
-        exe: ExecutableFile,
-        file: File,
-        ip: u64,
-        sp: u64,
-        args: [u64; 6],
-        ip_mem: Option<[u8; 64]>,
-        sp_mem: Option<[u8; 64]>,
-        memmap: Option<Vec<SydMemoryMap>>,
-    ) {
-        let result = ExecResult {
-            exe,
-            file,
-            ip,
-            sp,
-            args,
-            ip_mem,
-            sp_mem,
-            memmap,
-        };
+    pub(crate) fn add_exec(&self, pid: Pid, exe: ExecutableFile, file: File) {
+        let result = ExecResult { exe, file };
 
         self.sysres_map
             .trace_execv
@@ -311,6 +324,7 @@ impl<'a> WorkerCache<'a> {
         // Retire TGID from signal maps.
         self.retire_sig_handle(pid);
         self.retire_sig_restart(pid);
+        self.retire_ptrace(pid);
 
         // Remove preexisting error record for pid.
         let _ = self.get_error(pid);
@@ -405,20 +419,3 @@ impl WorkerData {
         val & WORKER_BUSY_MASK
     }
 }
-
-// [inode,(pid,path)] map of unix binds.
-// Path is only used for UNIX domain sockets.
-//
-// SAFETY:
-// 1. /proc/net/unix only gives inode information,
-//    and does not include information on device id
-//    or mount id so unfortunately we cannot check
-//    for that here.
-// 2. Pid is used for SO_PEERCRED getsockopt(2).
-#[derive(Copy, Clone)]
-pub(crate) struct UnixVal {
-    pub(crate) pid: Pid,
-    pub(crate) addr: Option<UnixAddr>,
-    pub(crate) peer: Option<UnixAddr>,
-}
-pub(crate) type UnixMap = Arc<RwLock<SydHashMap<u64, UnixVal>>>;
